@@ -1,23 +1,25 @@
 (* camlp5r *)
 (* pa_op.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "pa_extend.cmo";
 #load "q_MLast.cmo";
 
+open Asttools;
 open Exparser;
 open Pcaml;
 
 (* Syntax extensions in Ocaml grammar *)
 
 EXTEND
-  GLOBAL: expr ipatt;
+  GLOBAL: expr ipatt ext_attributes;
   expr: LEVEL "expr1"
     [ [ "parser"; po = OPT ipatt; OPT "|"; pcl = LIST1 parser_case SEP "|" ->
-          <:expr< $cparser loc po pcl$ >>
-      | "match"; e = SELF; "with"; "parser"; po = OPT ipatt; OPT "|";
+          <:expr< $cparser loc (po, pcl)$ >>
+      | "match"; (ext,attrs) = ext_attributes; e = SELF; "with"; "parser"; po = OPT ipatt; OPT "|";
         pcl = LIST1 parser_case SEP "|" ->
-          <:expr< $cparser_match loc e po pcl$ >> ] ]
+          expr_to_inline <:expr< $cparser_match loc e (po, pcl)$ >> ext attrs
+      ] ]
   ;
   parser_case:
     [ [ "[<"; sp = stream_patt; ">]"; po = OPT ipatt; "->"; e = expr ->
@@ -32,7 +34,8 @@ EXTEND
       | -> [] ] ]
   ;
   stream_patt_kont:
-    [ [ spc = stream_patt_comp_err -> [spc]
+    [ RIGHTA
+      [ spc = stream_patt_comp_err -> [spc]
       | spc = stream_patt_comp_err; ";" -> [spc]
       | spc = stream_patt_comp_err; ";"; sp = stream_patt_kont -> [spc :: sp]
       | spc = stream_patt_let; sp = stream_patt_kont -> [spc :: sp] ] ]
@@ -67,7 +70,8 @@ EXTEND
           <:expr< $cstream loc sel$ >> ] ]
   ;
   stream_expr_comp_list:
-    [ [ se = stream_expr_comp; ";"; sel = SELF -> [se :: sel]
+    [ RIGHTA
+      [ se = stream_expr_comp; ";"; sel = SELF -> [se :: sel]
       | se = stream_expr_comp; ";" -> [se]
       | se = stream_expr_comp -> [se] ] ]
   ;
