@@ -67,7 +67,7 @@ value rec ident len =
 value identifier kwt s =
   let con =
     try do {
-      (Hashtbl.find kwt s : unit);
+      ignore(Hashtbl.find kwt s : string);
       ""
     }
     with
@@ -523,7 +523,7 @@ value lexer_using kwt (con, prm) =
     "LIDENT" | "NL" | "QUOT" | "SPACEDOT" | "STRING" | "UIDENT" →
       ()
   | "ANTIQUOT" | "ANTIQUOT_LOC" → ()
-  | "" → try Hashtbl.find kwt prm with [ Not_found → Hashtbl.add kwt prm () ]
+  | "" → try do { Hashtbl.find kwt prm ; () } with [ Not_found → Hashtbl.add kwt prm prm ]
   | _ →
       raise
         (Plexing.Error
@@ -545,7 +545,7 @@ value lexer_gmake () =
   {Plexing.tok_func = Plexing.lexer_func_of_parser (lexer2 kwt);
    Plexing.tok_using = lexer_using kwt; Plexing.tok_removing = fun [];
    Plexing.tok_match = Plexing.default_match; Plexing.tok_text = lexer_text;
-   Plexing.tok_comm = None}
+   Plexing.tok_comm = None; Plexing.kwds = kwt }
 ;
 
 (* Building AST *)
