@@ -1,13 +1,15 @@
 (* camlp5r *)
 (* gramext.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 open Printf;
 
 type parser_t 'a = Stream.t 'a -> Obj.t;
+type fparser_t 'a = Fstream.t 'a -> option (Obj.t * Fstream.t 'a);
 type bparser_t 'a = Fstream.bp 'a Obj.t;
 
-type parse_algorithm = [ Predictive | Backtracking | DefaultAlgorithm ];
+type parse_algorithm =
+  [ Predictive | Functional | Backtracking | DefaultAlgorithm ];
 
 type grammar 'te =
   { gtokens : Hashtbl.t Plexing.pattern (ref int);
@@ -21,8 +23,10 @@ type g_entry 'te =
     elocal : bool;
     estart : mutable int -> parser_t 'te;
     econtinue : mutable int -> int -> Obj.t -> parser_t 'te;
-    bstart : mutable int -> bparser_t 'te;
-    bcontinue : mutable int -> int -> Obj.t -> bparser_t 'te;
+    fstart : mutable int -> err_fun -> fparser_t 'te;
+    fcontinue : mutable int -> int -> Obj.t -> err_fun -> fparser_t 'te;
+    bstart : mutable int -> err_fun -> bparser_t 'te;
+    bcontinue : mutable int -> int -> Obj.t -> err_fun -> bparser_t 'te;
     edesc : mutable g_desc 'te }
 and g_desc 'te =
   [ Dlevels of list (g_level 'te)
@@ -35,7 +39,6 @@ and g_level 'te =
 and g_assoc = [ NonA | RightA | LeftA ]
 and g_symbol 'te =
   [ Sfacto of g_symbol 'te
-  | Smeta of string and list (g_symbol 'te) and Obj.t
   | Snterm of g_entry 'te
   | Snterml of g_entry 'te and string
   | Slist0 of g_symbol 'te
@@ -46,17 +49,18 @@ and g_symbol 'te =
   | Sflag of g_symbol 'te
   | Sself
   | Snext
+  | Scut
   | Stoken of Plexing.pattern
   | Stree of g_tree 'te
   | Svala of list string and g_symbol 'te ]
 and g_action = Obj.t
 and g_tree 'te =
   [ Node of g_node 'te
-  | LocAct of g_action and list g_action
+  | LocAct of (string * g_action) and list (string * g_action)
   | DeadEnd ]
 and g_node 'te =
   { node : g_symbol 'te; son : g_tree 'te; brother : g_tree 'te }
-;
+and err_fun = unit -> string;
 
 type position =
   [ First
@@ -77,8 +81,8 @@ value rec derive_eps =
   | Sfacto s -> derive_eps s
   | Stree t -> tree_derive_eps t
   | Svala _ s -> derive_eps s
-  | Smeta _ _ _ | Slist1 _ | Slist1sep _ _ _ | Snterm _ | Snterml _ _ |
-    Snext | Sself | Stoken _ ->
+  | Slist1 _ | Slist1sep _ _ _ | Snterm _ | Snterml _ _ |
+    Snext | Sself | Scut | Stoken _ ->
       False ]
 and tree_derive_eps =
   fun
@@ -101,21 +105,22 @@ value rec eq_symbol s1 s2 =
   | (Sflag s1, Sflag s2) -> eq_symbol s1 s2
   | (Sopt s1, Sopt s2) -> eq_symbol s1 s2
   | (Svala ls1 s1, Svala ls2 s2) -> ls1 = ls2 && eq_symbol s1 s2
-  | (Stree _, Stree _) -> False
+  | (Stree t1, Stree t2) -> eq_tree t1 t2
   | (Sfacto (Stree t1), Sfacto (Stree t2)) ->
       (* The only goal of the node 'Sfacto' is to allow tree comparison
          (therefore factorization) without looking at the semantic
          actions; allow factorization of rules like "SV foo" which are
          actually expanded into a tree. *)
-      eq_tree t1 t2 where rec eq_tree t1 t2 =
+      eq_tree t1 t2
+  | _ -> s1 = s2 ]
+and eq_tree t1 t2 =
         match (t1, t2) with
         [ (Node n1, Node n2) ->
             eq_symbol n1.node n2.node && eq_tree n1.son n2.son &&
             eq_tree n1.brother n2.brother
-        | (LocAct _ _, LocAct _ _) -> True
+        | (LocAct (hash1, _) _, LocAct (hash2, _) _) -> hash1 = hash2
         | (DeadEnd, DeadEnd) -> True
         | _ -> False ]
-  | _ -> s1 = s2 ]
 ;
 
 value is_before s1 s2 =
@@ -159,6 +164,10 @@ value insert_tree entry_name gsymbols action tree =
         if eq_symbol s s1 then
           let t = Node {node = s1; son = insert sl son; brother = bro} in
           Some t
+        else if s = Scut then
+          try_insert s sl (Node {node = s; son = tree; brother = DeadEnd})
+        else if s1 = Scut then
+          try_insert s1 [s :: sl] tree
         else if is_before s1 s || derive_eps s && not (derive_eps s1) then
           let bro =
             match try_insert s sl bro with
@@ -182,7 +191,7 @@ value insert_tree entry_name gsymbols action tree =
 value srules rl =
   let t =
     List.fold_left
-      (fun tree (symbols, action) -> insert_tree "" symbols action tree)
+      (fun tree (symbols, hash, action) -> insert_tree "" symbols (hash, action) tree)
       DeadEnd rl
   in
   Stree t
@@ -207,7 +216,6 @@ and token_exists_in_tree f =
 and token_exists_in_symbol f =
   fun
   [ Sfacto sy -> token_exists_in_symbol f sy
-  | Smeta _ syl _ -> List.exists (token_exists_in_symbol f) syl
   | Slist0 sy -> token_exists_in_symbol f sy
   | Slist0sep sy sep _ ->
       token_exists_in_symbol f sy || token_exists_in_symbol f sep
@@ -219,7 +227,7 @@ and token_exists_in_symbol f =
   | Stoken tok -> f tok
   | Stree t -> token_exists_in_tree f t
   | Svala _ sy -> token_exists_in_symbol f sy
-  | Snterm _ | Snterml _ _ | Snext | Sself -> False ]
+  | Snterm _ | Snterml _ _ | Snext | Sself | Scut -> False ]
 ;
 
 value insert_level entry_name e1 symbols action slev =
@@ -353,7 +361,6 @@ Error: entries \"%s\" and \"%s\" do not belong to the same grammar.\n"
       }
       else ()
   | Sfacto s -> check_gram entry s
-  | Smeta _ sl _ -> List.iter (check_gram entry) sl
   | Slist0sep s t _ -> do { check_gram entry t; check_gram entry s }
   | Slist1sep s t _ -> do { check_gram entry t; check_gram entry s }
   | Slist0 s -> check_gram entry s
@@ -362,7 +369,7 @@ Error: entries \"%s\" and \"%s\" do not belong to the same grammar.\n"
   | Sflag s -> check_gram entry s
   | Stree t -> tree_check_gram entry t
   | Svala _ s -> check_gram entry s
-  | Snext | Sself | Stoken _ -> () ]
+  | Snext | Sself | Scut | Stoken _ -> () ]
 and tree_check_gram entry =
   fun
   [ Node {node = n; brother = bro; son = son} -> do {
@@ -389,7 +396,6 @@ value insert_tokens gram symbols =
   let rec insert =
     fun
     [ Sfacto s -> insert s
-    | Smeta _ sl _ -> List.iter insert sl
     | Slist0 s -> insert s
     | Slist1 s -> insert s
     | Slist0sep s t _ -> do { insert s; insert t }
@@ -411,7 +417,7 @@ value insert_tokens gram symbols =
         in
         incr r
       }
-    | Snterm _ | Snterml _ _ | Snext | Sself -> () ]
+    | Snterm _ | Snterml _ _ | Snext | Sself | Scut -> () ]
   and tinsert =
     fun
     [ Node {node = s; brother = bro; son = son} -> do {
@@ -443,12 +449,12 @@ value levels_of_rules entry position rules =
            let lev = make_lev lname assoc in
            let lev =
              List.fold_left
-               (fun lev (symbols, action) -> do {
+               (fun lev (symbols, hash, action) -> do {
                   let symbols = List.map (change_to_self entry) symbols in
                   List.iter (check_gram entry) symbols;
                   let (e1, symbols) = get_initial entry symbols in
                   insert_tokens entry.egram symbols;
-                  insert_level entry.ename e1 symbols action lev
+                  insert_level entry.ename e1 symbols (hash, action) lev
                 })
                lev level
            in
@@ -538,7 +544,6 @@ value rec decr_keyw_use gram =
       else ()
     }
   | Sfacto s -> decr_keyw_use gram s
-  | Smeta _ sl _ -> List.iter (decr_keyw_use gram) sl
   | Slist0 s -> decr_keyw_use gram s
   | Slist1 s -> decr_keyw_use gram s
   | Slist0sep s1 s2 _ -> do { decr_keyw_use gram s1; decr_keyw_use gram s2 }
@@ -547,7 +552,7 @@ value rec decr_keyw_use gram =
   | Sflag s -> decr_keyw_use gram s
   | Stree t -> decr_keyw_use_in_tree gram t
   | Svala _ s -> decr_keyw_use gram s
-  | Sself | Snext | Snterm _ | Snterml _ _ -> () ]
+  | Sself | Snext | Scut | Snterm _ | Snterml _ _ -> () ]
 and decr_keyw_use_in_tree gram =
   fun
   [ DeadEnd | LocAct _ _ -> ()
