@@ -1,26 +1,27 @@
 (* camlp5r *)
 (* plexer.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "pa_lexer.cmo";
 
 open Versdep;
 
+value simplest_raw_strings = ref False ;
 value no_quotations = ref False;
 value error_on_unknown_keywords = ref False;
 
 value dollar_for_antiquotation = ref True;
 value specific_space_dot = ref False;
-value dot_newline_is = ref ".";
 
 value force_antiquot_loc = ref False;
 
 type context =
   { after_space : mutable bool;
+    simplest_raw_strings : bool ;
     dollar_for_antiquotation : bool;
     specific_space_dot : bool;
-    dot_newline_is : string;
     find_kwd : string -> string;
+    is_kwd : string -> bool;
     line_cnt : int -> char -> unit;
     set_line_nb : unit -> unit;
     make_lined_loc : (int * int) -> string -> Ploc.t }
@@ -40,7 +41,7 @@ value keyword_or_error ctx loc s =
 
 value rev_implode l =
   let s = string_create (List.length l) in
-  loop (String.length s - 1) l where rec loop i =
+  bytes_to_string (loop (string_length s - 1) l) where rec loop i =
     fun
     [ [c :: l] -> do { string_unsafe_set s i c; loop (i - 1) l }
     | [] -> s ]
@@ -113,14 +114,19 @@ value rec ident =
   [ [ 'A'-'Z' | 'a'-'z' | '0'-'9' | '_' | ''' | misc_letter ] ident! | ]
 ;
 
-value rec ident2 =
+value rec ident2_or other =
   lexer
   [ [ '!' | '?' | '~' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' |
-      '%' | '.' | ':' | '<' | '>' | '|' | '$' | misc_punct ]
-      ident2!
+      '%' | '.' | ':' | '<' | '>' | '|' | '$' | other | misc_punct ]
+      (ident2_or other)!
   | ]
 ;
 
+value ident2 = ident2_or (fun buf strm -> raise Stream.Failure)
+;
+value hash_follower_chars = ident2_or (lexer [ '#' ])
+;
+
 value rec ident3 =
   lexer
   [ [ '0'-'9' | 'A'-'Z' | 'a'-'z' | '_' | '!' | '%' | '&' | '*' | '+' | '-' |
@@ -175,6 +181,48 @@ value number =
   | decimal_digits_under end_integer! ]
 ;
 
+(*
+let hex_float_literal =
+  '0' ['x' 'X']
+  ['0'-'9' 'A'-'F' 'a'-'f'] ['0'-'9' 'A'-'F' 'a'-'f' '_']*
+  ('.' ['0'-'9' 'A'-'F' 'a'-'f' '_']* )?
+  (['p' 'P'] ['+' '-']? ['0'-'9'] ['0'-'9' '_']* )?
+let literal_modifier = ['G'-'Z' 'g'-'z']
+
+let hex_float_literal =
+  '0' ['x' 'X']
+  ['0'-'9' 'A'-'F' 'a'-'f'] ['0'-'9' 'A'-'F' 'a'-'f' '_']*
+  ('.' ['0'-'9' 'A'-'F' 'a'-'f' '_']* )?
+  (['p' 'P'] ['+' '-']? ['0'-'9'] ['0'-'9' '_']* )?
+let literal_modifier = ['G'-'Z' 'g'-'z']
+*)
+
+(* hex_digits* *)
+value rec hex_digits_under_star =
+  lexer [ [ '0'-'9' | 'a'-'f' | 'A'-'F' | '_' ] hex_digits_under_star! | ]
+;
+value rec hex_under_integer =
+  lexer [ [ '0'-'9' | 'a'-'f' | 'A'-'F' ] hex_digits_under_star ]
+;
+value rec decimal_under_integer =
+  lexer [ [ '0'-'9' | ] decimal_digits_under! ]
+;
+
+value hex_exponent_part =
+  lexer
+  [ [ 'p' | 'P' ] [ '+' | '-' | ]
+    decimal_under_integer! ]
+;
+
+value hex_number =
+  lexer
+  [ hex_under_integer '.' hex_digits_under_star! hex_exponent_part -> ("FLOAT", $buf)
+  | hex_under_integer '.' hex_digits_under_star! -> ("FLOAT", $buf)
+  | hex_under_integer hex_exponent_part -> ("FLOAT", $buf)
+  | hex_under_integer exponent_part -> ("FLOAT", $buf)
+  | hex_under_integer end_integer! ]
+;
+
 value char_after_bslash =
   lexer
   [ "'"/
@@ -192,29 +240,30 @@ value any ctx buf =
   parser bp [: `c :] -> do { ctx.line_cnt bp c; $add c }
 ;
 
+value rec skiplws = lexer [
+  ' '/ skiplws!
+| '\t'/ skiplws!
+|
+]
+;
+
+value incrline ctx buf =
+  parser bp [: :] -> do {
+    ctx.line_cnt (bp-1) '\n'
+  ; buf
+  }
+;
+
 value rec string ctx bp =
   lexer
   [ "\""/
+  | "\\"/ ?= [ "\n" ] "\n"/ (incrline ctx)! skiplws! (string ctx bp)!
+  | "\\"/ ?= [ "\n" | " " ] (any ctx) (string ctx bp)!
   | "\\" (any ctx) (string ctx bp)!
   | (any ctx) (string ctx bp)!
   | -> err ctx (bp, $pos) "string not terminated" ]
 ;
 
-value comment ctx bp =
-  comment where rec comment =
-    lexer
-    [ "*)"
-    | "*" comment!
-    | "(*" comment! comment!
-    | "(" comment!
-    | "\"" (string ctx bp)! [ -> $add "\"" ] comment!
-    | "'*)"
-    | "'*" comment!
-    | "'" (any ctx) comment!
-    | (any ctx) comment!
-    | -> err ctx (bp, $pos) "comment not terminated" ]
-;
-
 value rec quotation ctx bp =
   lexer
   [ ">>"/
@@ -265,6 +314,42 @@ value rec antiquot ctx bp =
 
 value antiloc bp ep s = Printf.sprintf "%d,%d:%s" bp ep s;
 
+
+value skip_to_next_colon s i =
+  loop (i + 1) where rec loop j =
+    if j = String.length s then (i, 0)
+    else
+      match s.[j] with
+      [ ':' -> (j, j - i - 1)
+      | 'a'..'z' | 'A'..'Z' | '0'..'9' | '!' | '_' -> loop (j + 1)
+      | _ -> (i, 0) ]
+;
+
+value parse_antiquot s =
+  try
+    let i = String.index s ':' in
+    let kind = String.sub s 0 i in
+    let name = String.sub s (i+1) (String.length s - (i+1)) in
+    Some (kind, name)
+  with [ Not_found | Failure _ -> None ]
+;
+
+value parse_antiloc s =
+  try
+    let i = String.index s ':' in
+    let (j, len) = skip_to_next_colon s i in
+    let kind = String.sub s (i + 1) len in
+    let loc =
+      let k = String.index s ',' in
+      let bp = int_of_string (String.sub s 0 k) in
+      let ep = int_of_string (String.sub s (k + 1) (i - k - 1)) in
+      Ploc.make_unlined (bp, ep)
+    in
+    Some (loc, kind, String.sub s (j + 1) (String.length s - j - 1))
+  with
+  [ Not_found | Failure _ -> None ]
+;
+
 value rec antiquot_loc ctx bp =
   lexer
   [ "$"/ -> antiloc bp $pos (":" ^ $buf)
@@ -312,7 +397,7 @@ value question ctx bp buf strm =
         ("ANTIQUOT", "?" ^ s)
     | [: :] ->
         match strm with lexer
-        [ ident2! -> keyword_or_error ctx (bp, $pos) $buf ] ]
+        [ hash_follower_chars! -> keyword_or_error ctx (bp, $pos) $buf ] ]
   else if force_antiquot_loc.val then
     match strm with parser
     [ [: `'$'; s = antiquot_loc ctx bp $empty; `':' :] ->
@@ -321,10 +406,10 @@ value question ctx bp buf strm =
         ("ANTIQUOT_LOC", "?" ^ s)
     | [: :] ->
         match strm with lexer
-        [ ident2! -> keyword_or_error ctx (bp, $pos) $buf ] ]
+        [ hash_follower_chars! -> keyword_or_error ctx (bp, $pos) $buf ] ]
   else
     match strm with lexer
-    [ ident2! -> keyword_or_error ctx (bp, $pos) $buf ]
+    [ hash_follower_chars! -> keyword_or_error ctx (bp, $pos) $buf ]
 ;
 
 value tilde ctx bp buf strm =
@@ -336,7 +421,7 @@ value tilde ctx bp buf strm =
         ("ANTIQUOT", "~" ^ s)
     | [: :] ->
         match strm with lexer
-        [ ident2! -> keyword_or_error ctx (bp, $pos) $buf ] ]
+        [ hash_follower_chars! -> keyword_or_error ctx (bp, $pos) $buf ] ]
   else if force_antiquot_loc.val then
     match strm with parser
     [ [: `'$'; s = antiquot_loc ctx bp $empty; `':' :] ->
@@ -345,22 +430,32 @@ value tilde ctx bp buf strm =
         ("ANTIQUOT_LOC", "~" ^ s)
     | [: :] ->
         match strm with lexer
-        [ ident2! -> keyword_or_error ctx (bp, $pos) $buf ] ]
+        [ hash_follower_chars! -> keyword_or_error ctx (bp, $pos) $buf ] ]
   else
     match strm with lexer
-    [ ident2! -> keyword_or_error ctx (bp, $pos) $buf ]
+    [ hash_follower_chars! -> keyword_or_error ctx (bp, $pos) $buf ]
 ;
 
-value tildeident =
+value tildeident ~{raw} ctx loc =
+  let check_kwd s =
+    if raw then s
+    else if ctx.is_kwd s then err ctx loc "illegal token in TILDEIDENT"
+    else s
+  in
   lexer
-  [ ":"/ -> ("TILDEIDENTCOLON", $buf)
-  | -> ("TILDEIDENT", $buf) ]
+  [ ":"/ -> ("TILDEIDENTCOLON", check_kwd $buf)
+  | -> ("TILDEIDENT", check_kwd $buf) ]
 ;
 
-value questionident =
+value questionident ~{raw} ctx loc =
+  let check_kwd s =
+    if raw then s
+    else if ctx.is_kwd s then err ctx loc "illegal token in QUESTIONIDENT"
+    else s
+  in
   lexer
-  [ ":"/ -> ("QUESTIONIDENTCOLON", $buf)
-  | -> ("QUESTIONIDENT", $buf) ]
+  [ ":"/ -> ("QUESTIONIDENTCOLON", check_kwd $buf)
+  | -> ("QUESTIONIDENT", check_kwd $buf) ]
 ;
 
 value rec linedir n s =
@@ -386,31 +481,208 @@ value rec any_to_nl =
   | ]
 ;
 
+value rec rawstring1 delimtok (ofs, delim) ctx buf =
+  parser bp [: `c ; strm :] -> do {
+    ctx.line_cnt bp c;
+    let buf = $add c in
+    if String.get delim ofs <> c then
+       if String.get delim 0 = c then
+         rawstring1 delimtok (1, delim) ctx buf strm
+       else
+         rawstring1 delimtok (0, delim) ctx buf strm
+    else if ofs+1 < String.length delim then
+      rawstring1 delimtok (ofs+1, delim) ctx buf strm
+    else
+      let s = $buf in
+      let slen = String.length s in do {
+      (delimtok, String.sub s 0 (slen - (String.length delim)))
+      }
+  }
+;
+
+value rec rawstring0 ctx bp buf =
+  parser bp [
+    [: `'|' ; strm :] -> do {
+      rawstring1 $buf (0, "|" ^ $buf ^ "}") ctx $empty strm
+    }
+  | [: `('a'..'z' | '_' as c) ; strm :] -> do {
+      rawstring0 ctx bp ($add c) strm
+    }
+  ]
+;
+
+value add_string buf s =
+  let slen = String.length s in
+  let rec addrec buf i =
+    if i = slen then buf
+    else addrec ($add (String.get s i)) (i+1)
+  in addrec buf 0
+;
+
+(*
+ * This predicate checks that the stream contains a valid raw-string starter.  
+ * The definition of "valid raw string starter" depends on the value of 
+ * the variable [simplest_raw_strings]: if it is [False], then a valid
+ * raw-string starter is "[:alpha:]+|"; if it is [True], a valid raw-string
+ * starter is "[:alpha:]*|".  [simplest_raw_strings] is set to True in
+ * original syntax.
+
+ * This predicate gets called when the main lexer has already seen a "{".
+*)
+value raw_string_starter_p ctx strm =
+  let rec predrec n =
+    match stream_peek_nth n strm with
+      [ None -> False
+      | Some ('a'..'z' | '_') ->
+         predrec (n+1)
+      | Some '|' when ctx.simplest_raw_strings || n > 1 -> True
+      | Some _ -> False ]
+  in predrec 1
+;
+
+value comment_rawstring ctx bp (buf : Plexing.Lexbuf.t) strm =
+  if not (raw_string_starter_p ctx strm) then
+    buf
+  else
+  let (delim, s) = rawstring0 ctx bp $empty strm in
+  let rs = Printf.sprintf "{%s|%s|%s}" delim s delim in
+  add_string buf rs
+;
+
+value comment ctx bp =
+  comment where rec comment =
+    lexer
+    [ "*)"
+    | "*" comment!
+    | "{" (comment_rawstring ctx bp)! comment!
+    | "(*" comment! comment!
+    | "(" comment!
+    | "\"" (string ctx bp)! [ -> $add "\"" ] comment!
+    | "'*)"
+    | "'*" comment!
+    | "'" (any ctx) comment!
+    | (any ctx) comment!
+    | -> err ctx (bp, $pos) "comment not terminated" ]
+;
+
+value keyword_or_error_or_rawstring ctx bp (loc,s) buf strm =
+  if not (raw_string_starter_p ctx strm) then
+    match stream_peek_nth 1 strm with [
+      Some '|' when not ctx.simplest_raw_strings -> do {
+        Stream.junk strm ;
+        keyword_or_error ctx loc "{|" ;
+      }
+    | _ -> keyword_or_error ctx loc "{"
+    ]
+  else
+    let (delim, s) = rawstring0 ctx bp $empty strm in
+    ("STRING", String.escaped s)
+;
+
+value zerobuf f buf strm =
+  f $empty strm
+;
+
+value rec ws =
+  lexer
+  [ [' '/ | '\t'/ | '\n'/] [ ws | ]
+  ]
+;
+
+value rec extattrident =
+  lexer
+  [ ident [ "." extattrident | ] ]
+;
+
+value quoted_extension1 ctx (bp, _) extid buf strm =
+  let (delim, s) = rawstring0 ctx bp $empty strm in
+  ("QUOTEDEXTENSION", extid^":"^(String.escaped s))
+;
+
+value quoted_extension0 ctx (bp, _) extid =
+  lexer
+  [ ws (zerobuf (quoted_extension1 ctx (bp, $pos) extid))
+  | (zerobuf (quoted_extension1 ctx (bp, $pos) extid))
+  ]
+;
+
+value quoted_extension ctx (bp, _) =
+  lexer [
+    extattrident (zerobuf (quoted_extension0 ctx (bp, $pos) $buf))
+  ]
+;
+value dotsymbolchar = lexer
+  [ '!' | '$' | '%' | '&' | '*' | '+' | '-' | '/' | ':' | '=' | '>' | '?' | '@' | '^' | '|' ]
+;
+value rec dotsymbolchar_star = lexer
+  [ dotsymbolchar dotsymbolchar_star | ]
+;
+value kwdopchar = lexer
+  [ '$' | '&' | '*' | '+' | '-' | '/' | '<' | '=' | '>' | '@' | '^' | '|' ]
+;
+
+value symbolchar = lexer
+  ['!' | '$' | '%' | '&' | '*' | '+' | '-' | '.' | '/' | ':' | '<' | '=' | '>' | '?' | '@' | '^' | '|' | '~']
+;
+value rec symbolchar_star = lexer
+  [ symbolchar symbolchar_star | ]
+;
+
+value word_operators ctx id = lexer
+  [ [ kwdopchar dotsymbolchar_star ->
+      ("", id ^ $buf)
+    | -> try ("", ctx.find_kwd id) with [ Not_found -> ("LIDENT", id) ]
+    ] ]
+;
+value keyword = fun ~{raw} ctx buf strm ->
+  let id = $buf in
+  if raw then
+    ("LIDENT", id)
+  else if id = "let" || id = "and" then word_operators ctx id $empty strm
+  else
+    try ("", ctx.find_kwd id) with [ Not_found -> ("LIDENT", id) ]
+;
+
+value dot ctx (bp, pos) buf strm =
+  match Stream.peek strm with [
+    None ->
+      let id =
+        if ctx.specific_space_dot && ctx.after_space then " ." else "."
+      in
+      keyword_or_error ctx (bp, pos) id
+
+  | _ -> match strm with lexer [ [ -> $add "." ] dotsymbolchar_star! -> keyword_or_error ctx (bp, $pos) $buf ]
+  ]
+;
+
+
 value next_token_after_spaces ctx bp =
   lexer
   [ 'A'-'Z' ident! ->
       let id = $buf in
       try ("", ctx.find_kwd id) with [ Not_found -> ("UIDENT", id) ]
   | greek_letter ident! -> ("GIDENT", $buf)
-  | [ 'a'-'z' | '_' | misc_letter ] ident! ->
-      let id = $buf in
-      try ("", ctx.find_kwd id) with [ Not_found -> ("LIDENT", id) ]
+  | [ 'a'-'z' | '_' | misc_letter ] ident! (keyword ~{raw=False} ctx)
   | '1'-'9' number!
   | "0" [ 'o' | 'O' ] (digits octal)!
-  | "0" [ 'x' | 'X' ] (digits hexa)!
+  | "0" [ 'x' | 'X' ] (hex_number)!
   | "0" [ 'b' | 'B' ] (digits binary)!
   | "0" number!
-  | "'"/ ?= [ '\\' 'a'-'z' 'a'-'z' ] -> keyword_or_error ctx (bp, $pos) "'"
+  | "'"/ ?= [ '\\' '#' ] -> keyword_or_error ctx (bp, $pos) "'"
   | "'"/ (char ctx bp) -> ("CHAR", $buf)
   | "'" -> keyword_or_error ctx (bp, $pos) "'"
   | "\""/ (string ctx bp)! -> ("STRING", $buf)
   | "$"/ (dollar ctx bp)!
-  | [ '!' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' | '%' ] ident2! ->
+  | [ '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' | '%' ] ident2! ->
       keyword_or_error ctx (bp, $pos) $buf
-  | "~"/ 'a'-'z' ident! tildeident!
-  | "~"/ '_' ident! tildeident!
+  | '!' hash_follower_chars! -> keyword_or_error ctx (bp, $pos) $buf
+  | "~\\#"/ 'a'-'z' ident! (tildeident ~{raw=True} ctx (bp, $pos))!
+  | "~\\#"/ '_' ident! (tildeident ~{raw=True} ctx (bp, $pos))!
+  | "~"/ 'a'-'z' ident! (tildeident ~{raw=False} ctx (bp, $pos))!
+  | "~"/ '_' ident! (tildeident ~{raw=False} ctx (bp, $pos))!
   | "~" (tilde ctx bp)
-  | "?"/ 'a'-'z' ident! questionident!
+  | "?\\#"/ 'a'-'z' ident! (questionident ~{raw=True} ctx (bp, $pos))!
+  | "?"/ 'a'-'z' ident! (questionident ~{raw=False} ctx (bp, $pos))!
   | "?" (question ctx bp)!
   | "<"/ (less ctx bp)!
   | ":]" -> keyword_or_error ctx (bp, $pos) $buf
@@ -425,17 +697,23 @@ value next_token_after_spaces ctx bp =
   | "|}" -> keyword_or_error ctx (bp, $pos) $buf
   | "|" ident2! -> keyword_or_error ctx (bp, $pos) $buf
   | "[" ?= [ "<<" | "<:" ] -> keyword_or_error ctx (bp, $pos) $buf
+  | "[@" -> keyword_or_error ctx (bp, $pos) $buf
+  | "[@@" -> keyword_or_error ctx (bp, $pos) $buf
+  | "[@@@" -> keyword_or_error ctx (bp, $pos) $buf
+  | "[%" -> keyword_or_error ctx (bp, $pos) $buf
+  | "[%%" -> keyword_or_error ctx (bp, $pos) $buf
   | "[|" -> keyword_or_error ctx (bp, $pos) $buf
   | "[<" -> keyword_or_error ctx (bp, $pos) $buf
   | "[:" -> keyword_or_error ctx (bp, $pos) $buf
   | "[" -> keyword_or_error ctx (bp, $pos) $buf
   | "{" ?= [ "<<" | "<:" ] -> keyword_or_error ctx (bp, $pos) $buf
-  | "{|" -> keyword_or_error ctx (bp, $pos) $buf
   | "{<" -> keyword_or_error ctx (bp, $pos) $buf
+  | "{%"/ (zerobuf (quoted_extension ctx (bp, $pos)))
   | "{:" -> keyword_or_error ctx (bp, $pos) $buf
-  | "{" -> keyword_or_error ctx (bp, $pos) $buf
+  | "{" (keyword_or_error_or_rawstring ctx bp ((bp, $pos),$buf))
   | ".." -> keyword_or_error ctx (bp, $pos) ".."
-  | "." ?= [ "\n" ] -> keyword_or_error ctx (bp, bp + 1) ctx.dot_newline_is
+  | "." dotsymbolchar symbolchar_star ->
+      keyword_or_error ctx (bp, $pos) $buf
   | "." ->
       let id =
         if ctx.specific_space_dot && ctx.after_space then " ." else "."
@@ -445,7 +723,9 @@ value next_token_after_spaces ctx bp =
   | ";" -> keyword_or_error ctx (bp, $pos) ";"
   | (utf8_equiv ctx bp)
   | misc_punct ident2! -> keyword_or_error ctx (bp, $pos) $buf
+  | "\\#"/ [ 'a'-'z' | '_' | misc_letter ] ident! (keyword ~{raw=True} ctx)
   | "\\"/ ident3! -> ("LIDENT", $buf)
+  | "#" hash_follower_chars! -> keyword_or_error ctx (bp, $pos) $buf
   | (any ctx) -> keyword_or_error ctx (bp, $pos) $buf ]
 ;
 
@@ -526,15 +806,15 @@ value next_token_fun ctx glexr (cstrm, s_line_nb, s_bol_pos) =
       err ctx (Stream.count cstrm, Stream.count cstrm + 1) str ]
 ;
 
-value func kwd_table glexr =
-  let ctx =
+value make_ctx kwd_table =
     let line_nb = ref 0 in
     let bol_pos = ref 0 in
     {after_space = False;
      dollar_for_antiquotation = dollar_for_antiquotation.val;
+     simplest_raw_strings = simplest_raw_strings.val ;
      specific_space_dot = specific_space_dot.val;
-     dot_newline_is = dot_newline_is.val;
      find_kwd = Hashtbl.find kwd_table;
+     is_kwd = Hashtbl.mem kwd_table;
      line_cnt bp1 c =
        match c with
        [ '\n' | '\r' -> do {
@@ -548,7 +828,9 @@ value func kwd_table glexr =
      };
      make_lined_loc loc comm =
        Ploc.make_loc Plexing.input_file.val line_nb.val bol_pos.val loc comm}
-  in
+;
+
+value func ctx kwd_table glexr =
   Plexing.lexer_func_of_parser (next_token_fun ctx glexr)
 ;
 
@@ -575,6 +857,11 @@ and check =
   | "|}"
   | "|" check_ident2!
   | "[" ?= [ "<<" | "<:" ]
+  | "[@"
+  | "[@@"
+  | "[@@@"
+  | "[%"
+  | "[%%"
   | "[|"
   | "[<"
   | "[:"
@@ -599,8 +886,10 @@ and check_ident2 =
     check_ident2! | ]
 ;
 
-value check_keyword s =
-  try check_keyword_stream (Stream.of_string s) with _ -> False
+value check_keyword ctx s =
+  if ctx.simplest_raw_strings && (s = "{|" || s = "|}") then False
+  else
+    try check_keyword_stream (Stream.of_string s) with _ -> False
 ;
 
 value error_no_respect_rules p_con p_prm =
@@ -613,11 +902,11 @@ value error_no_respect_rules p_con p_prm =
           " does not respect Plexer rules"))
 ;
 
-value using_token kwd_table (p_con, p_prm) =
+value using_token ctx kwd_table (p_con, p_prm) =
   match p_con with
   [ "" ->
       if not (hashtbl_mem kwd_table p_prm) then
-        if check_keyword p_prm then Hashtbl.add kwd_table p_prm p_prm
+        if check_keyword ctx p_prm then Hashtbl.add kwd_table p_prm p_prm
         else error_no_respect_rules p_con p_prm
       else ()
   | "LIDENT" ->
@@ -634,6 +923,7 @@ value using_token kwd_table (p_con, p_prm) =
         | _ -> () ]
   | "TILDEIDENT" | "TILDEIDENTCOLON" | "QUESTIONIDENT" |
     "QUESTIONIDENTCOLON" | "INT" | "INT_l" | "INT_L" | "INT_n" | "FLOAT" |
+    "QUOTEDEXTENSION" |
     "CHAR" | "STRING" | "QUOTATION" | "GIDENT" |
     "ANTIQUOT" | "ANTIQUOT_LOC" | "EOI" ->
       ()
@@ -724,30 +1014,31 @@ value tok_match =
       fun (con, prm) ->
         if con = "LIDENT" then
           if p_prm = "" || prm = p_prm then prm else raise Stream.Failure
-	else
+        else
           if con = "" && prm = p_prm then prm else raise Stream.Failure
   | ("UIDENT", p_prm) ->
       (* also treats the case when a UIDENT is also a keyword *)
       fun (con, prm) ->
         if con = "UIDENT" then
           if p_prm = "" || prm = p_prm then prm else raise Stream.Failure
-	else
+        else
           if con = "" && prm = p_prm then prm else raise Stream.Failure
   | tok -> Plexing.default_match tok ]
 ;
 
 value gmake () =
   let kwd_table = Hashtbl.create 301 in
+  let ctx = make_ctx kwd_table in
   let glexr =
     ref
      {Plexing.tok_func = fun []; tok_using = fun []; tok_removing = fun [];
-      tok_match = fun []; tok_text = fun []; tok_comm = None}
+      tok_match = fun []; tok_text = fun []; tok_comm = None; kwds = kwd_table }
   in
   let glex =
-    {Plexing.tok_func = func kwd_table glexr;
-     tok_using = using_token kwd_table;
+    {Plexing.tok_func = func ctx kwd_table glexr;
+     tok_using = using_token ctx kwd_table;
      tok_removing = removing_token kwd_table;
-     tok_match = tok_match; tok_text = text; tok_comm = None}
+     tok_match = tok_match; tok_text = text; tok_comm = None ; kwds = kwd_table }
   in
   do { glexr.val := glex; glex }
 ;
