@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* token.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 type pattern = Plexing.pattern;
 
@@ -8,7 +8,7 @@ exception Error of string;
 
 type location = Ploc.t;
 type location_function = int -> location;
-type lexer_func 'te = Stream.t char -> (Stream.t 'te * location_function);
+type lexer_func 'te = Stream.t char -> (Stream.t 'te * Plexing.Locations.t);
 
 type glexer 'te = Plexing.lexer 'te ==
   { tok_func : lexer_func 'te;
@@ -16,7 +16,9 @@ type glexer 'te = Plexing.lexer 'te ==
     tok_removing : pattern -> unit;
     tok_match : mutable pattern -> 'te -> string;
     tok_text : pattern -> string;
-    tok_comm : mutable option (list location) }
+    tok_comm : mutable option (list location);
+    kwds : Hashtbl.t string string
+   }
 ;
 
 value make_loc = Ploc.make_unlined;
