@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* pa_extend.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "pa_macro.cmo";
 #load "pa_extend.cmo";
@@ -42,11 +42,10 @@ and a_symbol 'e 'p =
   | ASnext of loc
   | ASnterm of loc and (string * 'e) and option string
   | ASopt of loc and a_symbol 'e 'p
-  | ASfold of loc and string and string and 'e and 'e and a_symbol 'e 'p
-      and option (a_symbol 'e 'p)
   | ASquot of loc and a_symbol 'e 'p
   | ASrules of loc and a_rules 'e 'p
   | ASself of loc
+  | AScut of loc
   | AStok of loc and string and option (a_string 'e)
   | ASvala of loc and a_symbol 'e 'p and list string
   | ASvala2 of loc and a_symbol 'e 'p and list string
@@ -66,12 +65,12 @@ type styp =
   | STquo of loc and string
   | STself of loc and string
   | STtyp of MLast.ctyp
+  | STnone
   | STvala of loc and styp ]
 ;
 
 type text 'e 'p =
   [ TXfacto of loc and text 'e 'p
-  | TXmeta of loc and string and list (text 'e 'p) and 'e and styp
   | TXlist of loc and lmin_len and text 'e 'p and option (text 'e 'p * bool)
   | TXnext of loc
   | TXnterm of loc and name 'e and option string
@@ -80,6 +79,7 @@ type text 'e 'p =
   | TXrules of loc and string and list (rule 'e 'p)
   | TXself of loc
   | TXtok of loc and string and 'e
+  | TXcut of loc
   | TXvala of loc and list string and text 'e 'p ]
 and entry 'e 'p =
   { name : name 'e; pos : option 'e; levels : list (level 'e 'p) }
@@ -234,8 +234,15 @@ module MetaAction =
     value mloc = <:expr< Ploc.dummy >>;
     value rec mexpr =
       fun
-      [ MLast.ExAcc loc e1 e2 ->
-          <:expr< MLast.ExAcc $mloc$ $mexpr e1$ $mexpr e2$ >>
+      [ MLast.ExLong loc li ->
+          <:expr< MLast.ExLong $mloc$ $mlongid li$ >>
+
+      | MLast.ExOpen loc li e ->
+          <:expr< MLast.ExOpen $mloc$ $mlongid li$ $mexpr e$ >>
+
+      | MLast.ExFle loc e lili ->
+          <:expr< MLast.ExFle $mloc$ $mexpr e$ $mvala mlongid_lident lili$ >>
+
       | MLast.ExApp loc e1 e2 ->
           <:expr< MLast.ExApp $mloc$ $mexpr e1$ $mexpr e2$ >>
       | MLast.ExAsr loc e ->
@@ -247,10 +254,12 @@ module MetaAction =
           <:expr< MLast.ExIfe $mloc$ $mexpr e1$ $mexpr e2$ $mexpr e3$ >>
       | MLast.ExInt loc s c ->
           <:expr< MLast.ExInt $mloc$ $mvala mstring s$ $str:c$ >>
+      | MLast.ExLab loc peoptl ->
+          <:expr< MLast.ExLab $mloc$ $mvala (mlist mpeopt) peoptl$ >>
       | MLast.ExFlo loc s -> <:expr< MLast.ExFlo $mloc$ $mvala mstring s$ >>
       | MLast.ExLet loc rf pel e ->
           let rf = mvala mbool rf in
-          <:expr< MLast.ExLet $mloc$ $rf$ $mvala (mlist mpe) pel$ $mexpr e$ >>
+          <:expr< MLast.ExLet $mloc$ $rf$ $mvala (mlist mpea) pel$ $mexpr e$ >>
       | MLast.ExLid loc s -> <:expr< MLast.ExLid $mloc$ $mvala mstring s$ >>
       | MLast.ExMat loc e pwel ->
           <:expr< MLast.ExMat $mloc$ $mexpr e$ $mvala (mlist mpwe) pwel$ >>
@@ -259,8 +268,8 @@ module MetaAction =
           <:expr< MLast.ExRec $mloc$ $pel$ $moption mexpr eo$ >>
       | MLast.ExSeq loc el ->
           <:expr< MLast.ExSeq $mloc$ $mvala (mlist mexpr) el$ >>
-      | MLast.ExSte loc e1 e2 ->
-          <:expr< MLast.ExSte $mloc$ $mexpr e1$ $mexpr e2$ >>
+      | MLast.ExSte loc s e1 e2 ->
+          <:expr< MLast.ExSte $mvala mstring s$ $mloc$ $mexpr e1$ $mvala (mlist mexpr) e2$ >>
       | MLast.ExStr loc s ->
           <:expr< MLast.ExStr $mloc$ $mvala mstring_escaped s$ >>
       | MLast.ExTry loc e pwel ->
@@ -269,12 +278,14 @@ module MetaAction =
           <:expr< MLast.ExTup $mloc$ $mvala (mlist mexpr) el$ >>
       | MLast.ExTyc loc e t ->
           <:expr< MLast.ExTyc $mloc$ $mexpr e$ $mctyp t$ >>
-      | MLast.ExUid loc s -> <:expr< MLast.ExUid $mloc$ $mvala mstring s$ >>
       | x -> not_impl "mexpr" x ]
     and mpatt =
       fun
-      [ MLast.PaAcc loc p1 p2 ->
-          <:expr< MLast.PaAcc $mloc$ $mpatt p1$ $mpatt p2$ >>
+      [ MLast.PaPfx loc li p ->
+          <:expr< MLast.PaPfx $mloc$ $mlongid li$ $mpatt p$ >>
+      | MLast.PaLong loc li <:vala< [] >> ->
+          <:expr< MLast.PaLong $mloc$ $mlongid li$ (Ploc.VaVal []) >>
+      | MLast.PaLong loc li _ -> assert False
       | MLast.PaAny loc -> <:expr< MLast.PaAny $mloc$ >>
       | MLast.PaApp loc p1 p2 ->
           <:expr< MLast.PaApp $mloc$ $mpatt p1$ $mpatt p2$ >>
@@ -289,23 +300,39 @@ module MetaAction =
           <:expr< MLast.PaTup $mloc$ $mvala (mlist mpatt) pl$ >>
       | MLast.PaTyc loc p t ->
           <:expr< MLast.PaTyc $mloc$ $mpatt p$ $mctyp t$ >>
-      | MLast.PaUid loc s -> <:expr< MLast.PaUid $mloc$ $mvala mstring s$ >>
       | x -> not_impl "mpatt" x ]
+    and mlongid =
+      fun
+      [ MLast.LiApp loc me1 me2 -> <:expr< MLast.LiApp $mloc$ $mlongid me1$ $mlongid me2$ >>
+      | MLast.LiAcc loc me1 uid -> <:expr< MLast.LiAcc $mloc$ $mlongid me1$ $mvala mstring uid$ >>
+      | MLast.LiUid loc s -> <:expr< MLast.LiUid $mloc$ $mvala mstring s$ >>
+      | x -> not_impl "mlongid" x
+      ]
+    and mlongid_lident = fun
+      [ (Some li, id) -> <:expr< (Some $mvala mlongid li$, $mvala mstring id$) >>
+      | (None, id) -> <:expr< (None, $mvala mstring id$) >>
+      ]
     and mctyp =
       fun
-      [ MLast.TyAcc loc t1 t2 ->
-          <:expr< MLast.TyAcc $mloc$ $mctyp t1$ $mctyp t2$ >>
+      [ MLast.TyAcc loc m1 t2 ->
+          <:expr< MLast.TyAcc $mloc$ $mlongid m1$ $mvala mstring t2$ >>
       | MLast.TyApp loc t1 t2 ->
           <:expr< MLast.TyApp $mloc$ $mctyp t1$ $mctyp t2$ >>
       | MLast.TyLid loc s -> <:expr< MLast.TyLid $mloc$ $mvala mstring s$ >>
       | MLast.TyQuo loc s -> <:expr< MLast.TyQuo $mloc$ $mvala mstring s$ >>
       | MLast.TyTup loc tl ->
           <:expr< MLast.TyTup $mloc$ $mvala (mlist mctyp) tl$ >>
-      | MLast.TyUid loc s -> <:expr< MLast.TyUid $mloc$ $mvala mstring s$ >>
       | x -> not_impl "mctyp" x ]
-    and mpe (p, e) = <:expr< ($mpatt p$, $mexpr e$) >>
+    and mpea (p, e, attrs) =
+      do { assert ([] = Pcaml.unvala attrs) ;
+      <:expr< ($mpatt p$, $mexpr e$, Ploc.VaVal []) >>
+      }
+    and mpe (p, e) =
+      <:expr< ($mpatt p$, $mexpr e$) >>
     and mpwe (p, w, e) =
-      <:expr< ($mpatt p$, $mvala (moption mexpr) w$, $mexpr e$) >>;
+      <:expr< ($mpatt p$, $mvala (moption mexpr) w$, $mexpr e$) >>
+    and mpeopt (p, eopt) =
+      <:expr< ($mpatt p$, $mvala (moption mexpr) eopt$) >>;
   end
 ;
 
@@ -408,10 +435,10 @@ value quot_expr psl e =
         [ <:expr< $uid:c$ >> ->
             let al = List.map loop al in
             <:expr< Qast.Node $str:c$ $mklistexp loc al$ >>
-        | <:expr< MLast.$uid:c$ >> ->
+        | <:expr< MLast . $uid:c$ >> ->
             let al = List.map loop al in
             <:expr< Qast.Node $str:c$ $mklistexp loc al$ >>
-        | <:expr< $uid:m$.$uid:c$ >> ->
+        | <:expr< $uid:m$ . $uid:c$ >> ->
             let al = List.map loop al in
             <:expr< Qast.Node $str:m ^ "." ^ c$ $mklistexp loc al$ >>
         | <:expr< $lid:f$ >> ->
@@ -429,7 +456,7 @@ value quot_expr psl e =
                  let lab =
                    match p with
                    [ <:patt< $lid:c$ >> -> <:expr< $str:c$ >>
-                   | <:patt< $_$.$lid:c$ >> -> <:expr< $str:c$ >>
+                   | MLast.PaPfx _ _ <:patt< $lid:c$ >> -> <:expr< $str:c$ >>
                    | _ -> raise Not_found ]
                  in
                  <:expr< ($lab$, $loop e$) >>)
@@ -440,15 +467,15 @@ value quot_expr psl e =
         [ Not_found -> e ]
     | <:expr< $lid:s$ >> ->
         if s = Ploc.name.val then <:expr< Qast.Loc >> else e
-    | <:expr< MLast.$uid:s$ >> -> <:expr< Qast.Node $str:s$ [] >>
-    | <:expr< $uid:m$.$uid:s$ >> -> <:expr< Qast.Node $str:m ^ "." ^ s$ [] >>
+    | <:expr< MLast . $uid:s$ >> -> <:expr< Qast.Node $str:s$ [] >>
+    | <:expr< $uid:m$ . $uid:s$ >> -> <:expr< Qast.Node $str:m ^ "." ^ s$ [] >>
     | <:expr< $uid:s$ >> -> <:expr< Qast.Node $str:s$ [] >>
     | <:expr< $str:s$ >> -> <:expr< Qast.Str $str:s$ >>
     | <:expr< ($list:el$) >> ->
         let el = List.map loop el in
         <:expr< Qast.Tuple $mklistexp loc el$ >>
     | <:expr< let $flag:r$ $list:pel$ in $e$ >> ->
-        let pel = List.map (fun (p, e) -> (p, loop e)) pel in
+        let pel = List.map (fun (p, e, attrs) -> (p, loop e, attrs)) pel in
         <:expr< let $flag:r$ $list:pel$ in $loop e$ >>
     | _ -> e ]
 ;
@@ -504,6 +531,7 @@ value rec make_ctyp styp tvar =
           (Stream.Error ("'" ^ x ^ "' illegal in anonymous entry level"))
       else <:ctyp< '$tvar$ >>
   | STtyp t -> t
+  | STnone -> failwith "make_ctyp: internal error"
   | STvala loc t -> <:ctyp< Ploc.vala $make_ctyp t tvar$ >> ]
 ;
 
@@ -518,24 +546,28 @@ value text_of_action loc psl rtvar act tvar =
   let txt =
     List.fold_left
       (fun txt ps ->
-         match ps.pattern with
-         [ None -> <:expr< fun _ -> $txt$ >>
-         | Some p ->
-             let t = make_ctyp ps.symbol.styp tvar in
-             let p =
-               match p with
-               [ <:patt< ($list:pl$) >> when quotify.val ->
-                   <:patt< $lid:pname_of_ptuple pl$ >>
-               | _ -> p ]
-             in
-             <:expr< fun ($p$ : $t$) -> $txt$ >> ])
+         match ps.symbol.styp with
+         | STnone -> txt
+         | st ->
+             match ps.pattern with
+             [ None -> <:expr< fun _ -> $txt$ >>
+             | Some p ->
+                 let t = make_ctyp st tvar in
+                 let p =
+                   match p with
+                   [ <:patt< ($list:pl$) >> when quotify.val ->
+                       <:patt< $lid:pname_of_ptuple pl$ >>
+                   | _ -> p ]
+                 in
+                 <:expr< fun ($p$ : $t$) -> $txt$ >> ]
+             end)
       e psl
   in
   let txt =
     if meta_action.val then <:expr< Obj.magic $MetaAction.mexpr txt$ >>
     else txt
   in
-  <:expr< Gramext.action $txt$ >>
+  <:expr< $txt$ >>
 ;
 
 value srules loc t rl tvar =
@@ -547,76 +579,84 @@ value srules loc t rl tvar =
     rl
 ;
 
+value is_cut =
+  fun
+  [ TXcut _ -> True
+  | _ -> False ]
+;
+
 value rec make_expr gmod tvar =
   fun
-  [ TXfacto loc t -> <:expr< Gramext.Sfacto $make_expr gmod tvar t$ >>
-  | TXmeta loc n tl e t ->
-      let el =
-        List.fold_right
-          (fun t el -> <:expr< [$make_expr gmod "" t$ :: $el$] >>) tl
-          <:expr< [] >>
-      in
-      <:expr<
-        Gramext.Smeta $str:n$ $el$ (Obj.repr ($e$ : $make_ctyp t tvar$)) >>
+  [ TXfacto loc t -> <:expr< $uid:gmod$.s_facto $make_expr gmod tvar t$ >>
   | TXlist loc min t ts ->
       let txt = make_expr gmod "" t in
       match (min, ts) with
-      [ (LML_0, None) -> <:expr< Gramext.Slist0 $txt$ >>
-      | (LML_1, None) -> <:expr< Gramext.Slist1 $txt$ >>
+      [ (LML_0, None) -> <:expr< $uid:gmod$.s_list0 $txt$ >>
+      | (LML_1, None) -> <:expr< $uid:gmod$.s_list1 $txt$ >>
       | (LML_0, Some (s, b)) ->
           let x = make_expr gmod tvar s in
           let b = if b then <:expr< True >> else <:expr< False >> in
-          <:expr< Gramext.Slist0sep $txt$ $x$ $b$ >>
+          <:expr< $uid:gmod$.s_list0sep $txt$ $x$ $b$ >>
       | (LML_1, Some (s, b)) ->
           let x = make_expr gmod tvar s in
           let b = if b then <:expr< True >> else <:expr< False >> in
-          <:expr< Gramext.Slist1sep $txt$ $x$ $b$ >> ]
-  | TXnext loc -> <:expr< Gramext.Snext >>
+          <:expr< $uid:gmod$.s_list1sep $txt$ $x$ $b$ >> ]
+  | TXnext loc -> <:expr< $uid:gmod$.s_next >>
   | TXnterm loc n lev ->
       match lev with
       [ Some lab ->
           <:expr<
-             Gramext.Snterml
-               ($uid:gmod$.Entry.obj
-                  ($n.expr$ : $uid:gmod$.Entry.e '$n.tvar$))
+             $uid:gmod$.s_nterml
+               ($n.expr$ : $uid:gmod$.Entry.e '$n.tvar$)
                $str:lab$ >>
       | None ->
-          if n.tvar = tvar then <:expr< Gramext.Sself >>
+          if n.tvar = tvar then <:expr< $uid:gmod$.s_self >>
           else
             <:expr<
-               Gramext.Snterm
-                 ($uid:gmod$.Entry.obj
-                    ($n.expr$ : $uid:gmod$.Entry.e '$n.tvar$)) >> ]
-  | TXopt loc t -> <:expr< Gramext.Sopt $make_expr gmod "" t$ >>
-  | TXflag loc t -> <:expr< Gramext.Sflag $make_expr gmod "" t$ >>
+               $uid:gmod$.s_nterm
+                 ($n.expr$ : $uid:gmod$.Entry.e '$n.tvar$) >> ]
+  | TXopt loc t -> <:expr< $uid:gmod$.s_opt $make_expr gmod "" t$ >>
+  | TXflag loc t -> <:expr< $uid:gmod$.s_flag $make_expr gmod "" t$ >>
   | TXrules loc s rl ->
       let rl = srules loc s rl "" in
-      <:expr< Gramext.srules $make_expr_rules loc gmod rl ""$ >>
-  | TXself loc -> <:expr< Gramext.Sself >>
-  | TXtok loc s e -> <:expr< Gramext.Stoken ($str:s$, $e$) >>
+      <:expr< $uid:gmod$.s_rules $make_expr_rules loc gmod rl ""$ >>
+  | TXself loc -> <:expr< $uid:gmod$.s_self >>
+  | TXcut loc -> assert False
+  | TXtok loc s e -> <:expr< $uid:gmod$.s_token ($str:s$, $e$) >>
   | TXvala loc al t ->
       let al = make_list loc (fun s -> <:expr< $str:s$ >>) al in
-      <:expr< Gramext.Svala $al$ $make_expr gmod "" t$ >> ]
+      <:expr< $uid:gmod$.s_vala $al$ $make_expr gmod "" t$ >> ]
 and make_expr_rules loc gmod rl tvar =
   List.fold_left
     (fun txt (sl, ac) ->
        let sl =
-         List.fold_right
-           (fun t txt ->
-              let x = make_expr gmod tvar t in
-              <:expr< [$x$ :: $txt$] >>)
-           sl <:expr< [] >>
+         List.fold_left
+           (fun txt t ->
+              if is_cut t then <:expr< $uid:gmod$.r_cut $txt$ >>
+              else
+                let x = make_expr gmod tvar t in
+                <:expr< $uid:gmod$.r_next $txt$ $x$ >>)
+           <:expr< $uid:gmod$.r_stop >> sl
        in
-       <:expr< [($sl$, $ac$) :: $txt$] >>)
+       let hash = Printf.sprintf "%08x" (Hashtbl.hash (Reloc.expr (fun _ -> Ploc.dummy) 0 ac)) in
+       <:expr< [$uid:gmod$.production ($sl$, $str:hash$, $ac$) :: $txt$] >>)
     <:expr< [] >> rl
 ;
 
+value rec ident_of_longid = fun [
+  <:longident< $uid:s$ >> -> s
+| <:longident< $longid:li$ . $uid:s$ >> -> (ident_of_longid li) ^ "__" ^ s
+| li -> Ploc.raise (MLast.loc_of_longid li) (Failure "ident_of_longid: internal error in pa_extend")
+]
+;
+
 value rec ident_of_expr =
   fun
   [ <:expr< $lid:s$ >> -> s
   | <:expr< $uid:s$ >> -> s
-  | <:expr< $e1$ . $e2$ >> -> ident_of_expr e1 ^ "__" ^ ident_of_expr e2
-  | _ -> failwith "internal error in pa_extend" ]
+  | MLast.ExLong _ li -> ident_of_longid li
+  | e -> Ploc.raise (MLast.loc_of_expr e) (Failure "ident_of_expr: internal error in pa_extend")
+  ]
 ;
 
 value mk_name loc e = {expr = e; tvar = ident_of_expr e; loc = loc};
@@ -634,23 +674,6 @@ value slist loc min sep symb =
   TXlist loc min symb.text t
 ;
 
-value sfold loc n foldfun f e s =
-  let styp = STquo loc (new_type_var ()) in
-  let e = <:expr< Extfold.$lid:foldfun$ $f$ $e$ >> in
-  let t = STapp loc (STapp loc (STtyp <:ctyp< Extfold.t _ >>) s.styp) styp in
-  {used = s.used; text = TXmeta loc n [s.text] e t; styp = styp}
-;
-
-value sfoldsep loc n foldfun f e s sep =
-  let styp = STquo loc (new_type_var ()) in
-  let e = <:expr< Extfold.$lid:foldfun$ $f$ $e$ >> in
-  let t =
-    STapp loc (STapp loc (STtyp <:ctyp< Extfold.tsep _ >>) s.styp) styp
-  in
-  {used = s.used @ sep.used; text = TXmeta loc n [s.text; sep.text] e t;
-   styp = styp}
-;
-
 value mk_psymbol p s t =
   let symb = {used = []; text = s; styp = t} in
   {pattern = Some p; symbol = symb}
@@ -751,11 +774,6 @@ value rec symbol_of_a =
       let text = TXflag loc s.text in
       let styp = STlid loc "bool" in
       {used = s.used; text = text; styp = styp}
-  | ASfold loc n foldfun f e s sep ->
-      let s = symbol_of_a s in
-      match sep with
-      [ Some sep -> sfoldsep loc n foldfun f e s (symbol_of_a sep)
-      | None -> sfold loc n foldfun f e s ]
   | ASkeyw loc s ->
       let text = TXtok loc "" (string_of_a s) in
       {used = []; text = text; styp = STlid loc "string"}
@@ -789,6 +807,8 @@ value rec symbol_of_a =
        styp = STquo loc t}
   | ASself loc ->
       {used = []; text = TXself loc; styp = STself loc "SELF"}
+  | AScut loc ->
+      {used = []; text = TXcut loc; styp = STnone}
   | AStok loc s p ->
       let e =
         match p with
@@ -871,9 +891,11 @@ value expr_of_delete_rule loc gmod n sl =
   let n = mk_name2 n in
   let sl = List.map symbol_of_a sl in
   let sl =
-    List.fold_right
-      (fun s e -> <:expr< [$make_expr gmod "" s.text$ :: $e$] >>) sl
-      <:expr< [] >>
+    List.fold_left
+      (fun e s ->
+        if is_cut s.text then <:expr< $uid:gmod$.r_cut $e$ >>
+        else <:expr< $uid:gmod$.r_next $e$ $make_expr gmod "" s.text$ >>)
+      <:expr< $uid:gmod$.r_stop >> sl
   in
   (<:expr< $n.expr$ >>, sl)
 ;
@@ -933,7 +955,7 @@ value let_in_of_extend loc gmod functor_version gl el args =
       let globals =
         List.map
           (fun {expr = e; tvar = x; loc = loc} ->
-             (<:patt< _ >>, <:expr< ($e$ : $uid:gmod$.Entry.e '$x$) >>))
+             (<:patt< _ >>, <:expr< ($e$ : $uid:gmod$.Entry.e '$x$) >>, <:vala< [] >>))
           nl
       in
       let locals =
@@ -947,7 +969,8 @@ value let_in_of_extend loc gmod functor_version gl el args =
              (<:patt< $lid:i$ >>,
               <:expr<
                 (grammar_entry_create $str:i$ : $uid:gmod$.Entry.e '$x$)
-              >>))
+              >>,
+             <:vala< [] >>))
           ll
       in
       let e =
@@ -979,7 +1002,6 @@ value text_of_extend loc gmod gl el f =
       List.map
         (fun e ->
            let (ent, pos, txt) = text_of_entry e.name.loc gmod e in
-           let ent = <:expr< $uid:gmod$.Entry.obj $ent$ >> in
            let e = <:expr< ($ent$, $pos$, $txt$) >> in
            <:expr< let aux () = $f$ [$e$] in aux () >>)
         el
@@ -995,10 +1017,9 @@ value text_of_extend loc gmod gl el f =
       List.fold_right
         (fun e el ->
            let (ent, pos, txt) = text_of_entry e.name.loc gmod e in
-           let ent = <:expr< $uid:gmod$.Entry.obj $ent$ >> in
            let e =
              let loc = e.name.loc in
-             <:expr< ($ent$, $pos$, $txt$) >>
+             <:expr< $uid:gmod$.extension $ent$ $pos$ $txt$ >>
            in
            <:expr< [$e$ :: $el$] >>)
         el <:expr< [] >>
@@ -1015,7 +1036,7 @@ value text_of_functorial_extend loc gmod gl el =
       List.map
         (fun e ->
            let (ent, pos, txt) = text_of_entry e.name.loc gmod e in
-           let e = <:expr< $uid:gmod$.extend $ent$ $pos$ $txt$ >> in
+           let e = <:expr< $uid:gmod$.safe_extend $ent$ $pos$ $txt$ >> in
            if split_ext.val then <:expr< let aux () = $e$ in aux () >> else e)
         el
     in
@@ -1026,8 +1047,27 @@ value text_of_functorial_extend loc gmod gl el =
   let_in_of_extend loc gmod True gl el args
 ;
 
+value longname_of_raw_qualid (l, id) =
+  String.concat "__" (l@[id])
+;
+
+value expr_of_raw_qualid loc (l, id) =
+  match l with [
+    [ _ :: _ ] ->
+    let li = Asttools.longident_of_string_list loc l in
+    let vid = <:vala< id >> in
+    MLast.ExFle loc (MLast.ExLong loc li) <:vala< (None, vid) >>
+  | [] ->
+    <:expr< $lid:id$ >>
+  ]
+;
+
 open Pcaml;
 value symbol = Grammar.Entry.create gram "symbol";
+value rule = Grammar.Entry.create gram "rule";
+value rule_list = Grammar.Entry.create gram "rule_list";
+value level_list = Grammar.Entry.create gram "level_list";
+value level = Grammar.Entry.create gram "level";
 value semi_sep =
   if syntax_name.val = "Scheme" then
     Grammar.Entry.of_parser gram "'/'" (parser [: `("", "/") :] -> ())
@@ -1036,12 +1076,12 @@ value semi_sep =
 ;
 
 EXTEND
-  GLOBAL: expr symbol;
+  GLOBAL: expr symbol rule rule_list level level_list;
   expr: AFTER "top"
-    [ [ "EXTEND"; e = extend_body; "END" -> e
-      | "GEXTEND"; e = gextend_body; "END" -> e
-      | "DELETE_RULE"; e = delete_rule_body; "END" -> e
-      | "GDELETE_RULE"; e = gdelete_rule_body; "END" -> e ] ]
+    [ [ "EXTEND"; /; e = extend_body; "END" -> e
+      | "GEXTEND"; /; e = gextend_body; "END" -> e
+      | "DELETE_RULE"; /; e = delete_rule_body; "END" -> e
+      | "GDELETE_RULE"; /; e = gdelete_rule_body; "END" -> e ] ]
   ;
   extend_body:
     [ [ f = efunction; sl = OPT global;
@@ -1056,16 +1096,16 @@ EXTEND
   delete_rule_body:
     [ [ n = name; ":"; sl = LIST1 symbol SEP semi_sep ->
           let (e, b) = expr_of_delete_rule loc "Grammar" n sl in
-          <:expr< Grammar.delete_rule $e$ $b$ >> ] ]
+          <:expr< Grammar.safe_delete_rule $e$ $b$ >> ] ]
   ;
   gdelete_rule_body:
     [ [ g = UIDENT; n = name; ":"; sl = LIST1 symbol SEP semi_sep ->
           let (e, b) = expr_of_delete_rule loc g n sl in
-          <:expr< $uid:g$.delete_rule $e$ $b$ >> ] ]
+          <:expr< $uid:g$.safe_delete_rule $e$ $b$ >> ] ]
   ;
   efunction:
     [ [ UIDENT "FUNCTION"; ":"; f = qualid; semi_sep -> snd f
-      | -> <:expr< Grammar.extend >> ] ]
+      | -> <:expr< Grammar.safe_extend >> ] ]
   ;
   global:
     [ [ UIDENT "GLOBAL"; ":"; sl = LIST1 name; semi_sep -> sl ] ]
@@ -1159,12 +1199,20 @@ EXTEND
           AStok loc x (Some e)
       | e = string ->
           ASkeyw loc e
-      | i = UIDENT; "."; e = qualid;
+
+      | i = UIDENT; "."; (l, id) = raw_qualid;
+        lev = OPT [ UIDENT "LEVEL"; s = STRING -> s ] ->
+        let l = [i :: l] in
+        let qidname = longname_of_raw_qualid (l, id) in
+        let v = expr_of_raw_qualid loc (l, id) in
+        ASnterm loc (qidname, v) lev
+
+      | n = name;
         lev = OPT [ UIDENT "LEVEL"; s = STRING -> s ] ->
-          let v = <:expr< $uid:i$.$snd e$ >> in
-          ASnterm loc (i ^ "__" ^ fst e, v) lev
-      | n = name; lev = OPT [ UIDENT "LEVEL"; s = STRING -> s ] ->
           ASnterm loc n lev
+
+      | "/" ->
+          AScut loc
       | "("; s_t = SELF; ")" -> s_t ] ]
   ;
   pattern:
@@ -1182,12 +1230,18 @@ EXTEND
     [ [ e = qualid -> e ] ]
   ;
   qualid:
-    [ [ e1 = SELF; "."; e2 = SELF ->
-          (fst e1 ^ "__" ^ fst e2, <:expr< $snd e1$ . $snd e2$ >>) ]
-    | [ i = UIDENT ->
-          (i, <:expr< $uid:i$ >>)
-      | i = LIDENT ->
-          (i, <:expr< $lid:i$ >>) ] ]
+    [ [ (l, id) = raw_qualid ->
+        let qidname = longname_of_raw_qualid (l, id) in
+        let v = expr_of_raw_qualid loc (l, id) in
+        (qidname, v)
+      ]
+    ]
+  ;
+  raw_qualid:
+    [ [ l = LIST1 [ i = UIDENT ; "." -> i] ; id = LIDENT -> (l, id)
+      | id = LIDENT -> ([], id)
+      ]
+    ]
   ;
   string:
     [ [ s = STRING -> ATstring loc s
