@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* pa_macro.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 #load "pa_extend.cmo";
 #load "q_MLast.cmo";
@@ -101,6 +101,7 @@ Added statements:
 
 *)
 
+open Asttools;
 open Pcaml;
 open Printf;
 open Versdep;
@@ -126,18 +127,18 @@ value rec list_remove x =
 ;
 
 value oversion = do {
-  let v = string_copy Pconfig.ocaml_version in
-  for i = 0 to String.length v - 1 do {
-    match v.[i] with
+  let v = string_copy (bytes_of_string Pcaml.ocaml_version) in
+  for i = 0 to string_length v - 1 do {
+    match string_get v i with
     [ '0'..'9' | 'a'..'z' | 'A'..'Z' -> ()
     | _ -> string_set v i '_' ];
   };
-  v
+  bytes_to_string v
 };
 
 value oname =
-  if Pconfig.ocaml_name = "ocaml" then []
-  else [(string_uppercase Pconfig.ocaml_name, MvNone)]
+  if ocaml_name = "ocaml" then []
+  else [(string_uppercase ocaml_name, MvNone)]
 ;
 
 value defined =
@@ -161,8 +162,6 @@ value defined_version loc =
 
 value is_defined i =
   (i = "STRICT" && Pcaml.strict_mode.val) ||
-  (i = "COMPATIBLE_WITH_OLD_OCAML" &&
-   Pcaml.flag_compatible_old_versions_of_ocaml.val) ||
   List.mem_assoc i defined.val
 ;
 
@@ -190,7 +189,7 @@ value subst mloc env =
   loop where rec loop =
     fun
     [ <:expr< let $flag:rf$ $list:pel$ in $e$ >> ->
-        let pel = List.map (fun (p, e) -> (p, loop e)) pel in
+        let pel = List.map (fun (p, e, attrs) -> (p, loop e, attrs)) pel in
         <:expr< let $flag:rf$ $list:pel$ in $loop e$ >>
     | <:expr< if $e1$ then $e2$ else $e3$ >> ->
         <:expr< if $loop e1$ then $loop e2$ else $loop e3$ >>
@@ -207,14 +206,16 @@ value subst mloc env =
 value substp mloc env =
   loop where rec loop =
     fun
-    [ <:expr< $e1$ . $e2$ >> -> <:patt< $loop e1$ . $loop e2$ >>
+    [ <:expr< $uid:x$ >> ->
+        try <:patt< $anti:List.assoc x env$ >> with
+        [ Not_found -> <:patt< $uid:x$ >> ]
+    | <:expr:< $longid:li$ >> ->
+      <:patt< $longid:li$ >>
+
     | <:expr< $e1$ $e2$ >> -> <:patt< $loop e1$ $loop e2$ >>
     | <:expr< $lid:x$ >> ->
         try <:patt< $anti:List.assoc x env$ >> with
         [ Not_found -> <:patt< $lid:x$ >> ]
-    | <:expr< $uid:x$ >> ->
-        try <:patt< $anti:List.assoc x env$ >> with
-        [ Not_found -> <:patt< $uid:x$ >> ]
     | <:expr< $int:x$ >> -> <:patt< $int:x$ >>
     | <:expr< $chr:x$ >> -> <:patt< $chr:x$ >>
     | <:expr< $str:x$ >> -> <:patt< $str:x$ >>
@@ -234,7 +235,7 @@ value substt mloc env =
     [ <:ctyp< $t1$ -> $t2$ >> -> <:ctyp< $loop t1$ -> $loop t2$ >>
     | <:ctyp< $t1$ $t2$ >> -> <:ctyp< $loop t1$ $loop t2$ >>
     | <:ctyp< ($list:tl$) >> -> <:ctyp< ($list:List.map loop tl$) >>
-    | <:ctyp< $lid:x$ >> | <:ctyp< $uid:x$ >> as t ->
+    | <:ctyp< $lid:x$ >> as t ->
         try List.assoc x env with [ Not_found -> t ]
     | t -> t ]
 ;
@@ -403,8 +404,9 @@ value apply_directive loc n dp =
       Ploc.raise loc (Stream.Error msg) ]
 ;
 
+value dexpr = Grammar.Entry.create gram "dexpr";
 EXTEND
-  GLOBAL: expr patt str_item sig_item constructor_declaration match_case
+  GLOBAL: dexpr expr patt str_item sig_item constructor_declaration match_case
     label_declaration;
   str_item: FIRST
     [ [ x = str_macro_def ->
@@ -522,7 +524,7 @@ EXTEND
           let ep = string_of_int (Ploc.last_pos loc) in
           <:expr< ($int:bp$, $int:ep$) >> ] ]
   ;
-  patt:
+  patt: FIRST
     [ [ "IFDEF"; e = dexpr; "THEN"; p1 = SELF; p2 = else_patt; "END" ->
           if e then p1 else p2
       | "IFNDEF"; e = dexpr; "THEN"; p1 = SELF; p2 = else_patt; "END" ->
@@ -566,7 +568,8 @@ EXTEND
           if not e then x else Grammar.skip_item x ] ]
   ;
   else_match_case:
-    [ [ "ELSIFDEF"; e = dexpr; "THEN"; x = match_case; y = else_match_case ->
+    [ RIGHTA
+      [ "ELSIFDEF"; e = dexpr; "THEN"; x = match_case; y = else_match_case ->
           if e then x else y
       | "ELSIFDEF"; e = dexpr; "THEN"; x = match_case ->
           if e then x else Grammar.skip_item x
