@@ -1211,6 +1211,8 @@ EXTEND
       ]
     | NONA [ "exception"; (ext,attrs) = ext_attributes; p = patt →
         patt_to_inline loc <:patt< exception $p$ >> ext attrs
+      | "effect" ; p1 = SELF ; "," ; p2 = patt LEVEL "simple" ->
+         <:patt< effect $p1$ , $p2$ >>
       ]
     | NONA
       [ p1 = SELF; ".."; p2 = SELF → <:patt< $p1$ .. $p2$ >> ]
