@@ -1151,7 +1151,10 @@ EXTEND
         Qast.Node "PaAtt" [Qast.Loc; p1; attr]
       ]
     | NONA
-      [ "exception"; p = SELF → Qast.Node "PaExc" [Qast.Loc; p] ]
+      [ "exception"; p = SELF → Qast.Node "PaExc" [Qast.Loc; p]
+      | "effect" ; p1 = SELF ; "," ; p2 = patt LEVEL "simple" ->
+         Qast.Node "PaEff" [Qast.Loc; p1; p2]
+      ]
 
     | NONA
       [ p1 = SELF; ".."; p2 = SELF → Qast.Node "PaRng" [Qast.Loc; p1; p2] ]
@@ -1310,6 +1313,8 @@ Qast.Node "PaLong" [Qast.Loc; Qast.Node "LiUid" [Qast.Loc; (Qast.VaVal (Qast.Str
     [ LEFTA
       [ me1 = extended_longident ; "." ; i = SV LIDENT "lid" → 
           Qast.Node "TyAcc" [Qast.Loc; me1; i]
+      | me1 = extended_longident ; "." ; "(" ; t = ctyp ; ")" → 
+          Qast.Node "TyOpen" [Qast.Loc; me1; t]
       | i = SV LIDENT "lid" → 
           Qast.Node "TyLid" [Qast.Loc; i]
       ] 
