@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* plexing.mli,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 (** Lexing for Camlp5 grammars.
 
@@ -22,6 +22,16 @@ type pattern = string * string;;
 exception Error of string;;
     (** A lexing error exception to be used by lexers. *)
 
+(** location array, used by lexer to find locations based on token-index *)
+module Locations :
+  sig
+    type t = { locations : Ploc.t option array ref; overflow : bool ref };;
+    val create : unit -> t;;
+    val lookup : t -> int -> Ploc.t;;
+    val add : t -> int -> Ploc.t -> unit;;
+  end
+;;
+
 (** Lexer type *)
 
 type 'te lexer =
@@ -30,8 +40,9 @@ type 'te lexer =
     tok_removing : pattern -> unit;
     mutable tok_match : pattern -> 'te -> string;
     tok_text : pattern -> string;
-    mutable tok_comm : Ploc.t list option }
-and 'te lexer_func = char Stream.t -> 'te Stream.t * location_function
+    mutable tok_comm : Ploc.t list option;
+    kwds : (string, string) Hashtbl.t }
+and 'te lexer_func = char Stream.t -> 'te Stream.t * Locations.t
 and location_function = int -> Ploc.t;;
   (** The type of a function giving the location of a token in the
       source from the token number in the stream (starting from zero). *)
@@ -67,13 +78,19 @@ val lexer_func_of_parser :
        returning the next token and its location. The two references
        with the char stream contain the current line number and the
        position of the beginning of the current line. *)
+val lexer_func_of_ocamllex_located :
+  (Lexing.lexbuf -> 'te * Ploc.t) -> 'te lexer_func;;
+   (** A lexer function from a lexer created by [ocamllex],
+       but that returns camlp5 locations, too.  This can be used to get comments properly
+       transmitted to the parser.
+    *)
 val lexer_func_of_ocamllex : (Lexing.lexbuf -> 'te) -> 'te lexer_func;;
    (** A lexer function from a lexer created by [ocamllex] *)
 
 (** Function to build a stream and a location function *)
 
 val make_stream_and_location :
-  (unit -> 'te * Ploc.t) -> 'te Stream.t * location_function;;
+  (unit -> 'te * Ploc.t) -> 'te Stream.t * Locations.t;;
    (** General function *)
 
 (** Useful functions and values *)
