@@ -17,7 +17,8 @@ type 'te glexer =
       tok_removing : pattern -> unit;
       mutable tok_match : pattern -> 'te -> string;
       tok_text : pattern -> string;
-      mutable tok_comm : location list option }
+      mutable tok_comm : location list option;
+      kwds : (string, string) Hashtbl.t }
 ;;
 
 let make_loc = Ploc.make_unlined;;
