@@ -241,6 +241,7 @@ let ocaml_mkmty loc x =
 let ocaml_mkmod loc x =
   {pmod_desc = x; pmod_loc = loc; pmod_attributes = []}
 ;;
+let ocaml_mkconst loc x = x;;
 
 let ocaml_mkfield_inh ?(alg_attributes = []) loc x fl =
   {pof_desc = Oinherit x; pof_loc = loc; pof_attributes = alg_attributes} ::
@@ -255,18 +256,14 @@ let ocaml_mkfield_tag ?(alg_attributes = []) loc (lab, x) fl =
 let ocaml_mkfield_var loc = [];;
 
 let convert_camlp5_variance (va, inj) =
+  assert (inj = false);
   let va =
     match va with
       Some false -> Contravariant
     | Some true -> Covariant
-    | _ -> NoVariance
+    | _ -> Invariant
   in
-  let inj =
-    match inj with
-      true -> Injective
-    | false -> NoInjectivity
-  in
-  va, inj
+  va
 ;;
 
 let ocaml_ec_tuple ?(alg_attributes = []) loc s tyvars (x, rto) =
@@ -506,6 +503,10 @@ let ocaml_case (p, wo, loc, e) =
 
 (* *)
 
+let ocaml_ppat_effect p1 p2 =
+  failwith "Internal error: effect pattern not  available until Ocaml 5.3.0"
+;;
+
 let ocaml_pexp_function lab eo pel =
   match pel with
     [{pc_lhs = p; pc_guard = None; pc_rhs = {pexp_desc = Pexp_unreachable}}]
