@@ -47,6 +47,7 @@ let mkexp loc d = ocaml_mkexp (mkloc loc) d;;
 let mkmty loc d = ocaml_mkmty (mkloc loc) d;;
 let mksig loc d = {psig_desc = d; psig_loc = mkloc loc};;
 let mkmod loc d = ocaml_mkmod (mkloc loc) d;;
+let mkconst loc d = ocaml_mkconst (mkloc loc) d;;
 let mkstr loc d = {pstr_desc = d; pstr_loc = mkloc loc};;
 let mkfield_tag ~alg_attributes:alg_attributes loc d fl =
   ocaml_mkfield_tag ~alg_attributes:alg_attributes (mkloc loc) d fl
@@ -317,11 +318,12 @@ let exception_to_constructor_pattern =
 
 let rec mkrangepat loc c1 c2 =
   if c1 > c2 then mkrangepat loc c2 c1
-  else if c1 = c2 then mkpat loc (Ppat_constant (ocaml_pconst_char c1))
+  else if c1 = c2 then
+    mkpat loc (Ppat_constant (mkconst loc (ocaml_pconst_char c1)))
   else
     mkpat loc
       (Ppat_or
-         (mkpat loc (Ppat_constant (ocaml_pconst_char c1)),
+         (mkpat loc (Ppat_constant (mkconst loc (ocaml_pconst_char c1))),
           mkrangepat loc (Char.chr (Char.code c1 + 1)) c2))
 ;;
 
@@ -661,7 +663,7 @@ and ctyp =
         | MLast.TyQuo (_, s), t -> t, s
         | _ -> error loc "incorrect alias type"
       in
-      mktyp loc (Ptyp_alias (ctyp t, i))
+      mktyp loc (ocaml_ptyp_alias (mkloc loc) (ctyp t) i)
   | TyAny loc -> mktyp loc Ptyp_any
   | TyApp (loc, _, _) as f ->
       let (f, al) = ctyp_fa [] f in
@@ -731,6 +733,9 @@ and ctyp =
   | TyXtr (loc, _, _) -> error loc "bad ast TyXtr"
   | TyExten (loc, ebody) ->
       mktyp loc (ocaml_ptyp_extension (extension (uv ebody)))
+  | TyOpen (loc, li, t) ->
+      let li = longid_long_id li in
+      mktyp loc (ocaml_ptyp_open (mkloc loc) li (ctyp t))
 and meth_list loc fl v =
   match fl with
     [] -> if uv v then mkfield_var loc else []
@@ -862,6 +867,9 @@ and patt =
   | PaAnt (_, p) -> patt p
   | PaAny loc -> mkpat loc Ppat_any
   | PaExc (loc, p) -> let p = patt p in mkpat loc (ocaml_ppat_exception p)
+  | PaEff (loc, p1, p2) ->
+      let p1 = patt p1 in
+      let p2 = patt p2 in mkpat loc (ocaml_ppat_effect p1 p2)
   | PaApp (loc, _, _) as f0 ->
       let (f, al) = patt_fa [] f0 in
       begin match f with
@@ -911,10 +919,14 @@ and patt =
       end
   | PaChr (loc, s) ->
       mkpat loc
-        (Ppat_constant (ocaml_pconst_char (char_of_char_token loc (uv s))))
+        (Ppat_constant
+           (mkconst loc (ocaml_pconst_char (char_of_char_token loc (uv s)))))
   | PaInt (loc, s, c) ->
-      mkpat loc (Ppat_constant (pconst_of_const (mkintconst loc (uv s) c)))
-  | PaFlo (loc, s) -> mkpat loc (Ppat_constant (ocaml_pconst_float (uv s)))
+      mkpat loc
+        (Ppat_constant
+           (mkconst loc (pconst_of_const (mkintconst loc (uv s) c))))
+  | PaFlo (loc, s) ->
+      mkpat loc (Ppat_constant (mkconst loc (ocaml_pconst_float (uv s))))
   | PaLab (loc, _) -> error loc "labeled pattern not allowed here"
   | PaLaz (loc, p) ->
       begin match ocaml_ppat_lazy with
@@ -945,8 +957,9 @@ and patt =
   | PaStr (loc, s) ->
       mkpat loc
         (Ppat_constant
-           (ocaml_pconst_string (string_of_string_token loc (uv s))
-              (mkloc loc) None))
+           (mkconst loc
+              (ocaml_pconst_string (string_of_string_token loc (uv s))
+                 (mkloc loc) None)))
   | PaTup (loc, pl) -> mkpat loc (Ppat_tuple (List.map patt (uv pl)))
   | PaTyc (loc, p, t) -> mkpat loc (Ppat_constraint (patt p, ctyp t))
   | PaTyp (loc, lili) ->
@@ -1025,10 +1038,11 @@ and expr =
   | ExAnt (_, e) -> expr e
   | ExApp (loc, ExLid (_, "-"), ExInt (_, s, c)) ->
       let s = neg_string (uv s) in
-      mkexp loc (Pexp_constant (pconst_of_const (mkintconst loc s c)))
+      mkexp loc
+        (Pexp_constant (mkconst loc (pconst_of_const (mkintconst loc s c))))
   | ExApp (loc, ExLid (_, ("-" | "-.")), ExFlo (_, s)) ->
       let s = neg_string (uv s) in
-      mkexp loc (Pexp_constant (ocaml_pconst_float s))
+      mkexp loc (Pexp_constant (mkconst loc (ocaml_pconst_float s)))
   | ExApp (loc, _, _) as f ->
       let (f, al) = expr_fa [] f in
       let f =
@@ -1214,11 +1228,13 @@ and expr =
       end
   | ExChr (loc, s) ->
       mkexp loc
-        (Pexp_constant (ocaml_pconst_char (char_of_char_token loc (uv s))))
+        (Pexp_constant
+           (mkconst loc (ocaml_pconst_char (char_of_char_token loc (uv s)))))
   | ExCoe (loc, e, t1, t2) ->
       mkexp loc
         (ocaml_pexp_constraint (expr e) (option_map ctyp t1) (Some (ctyp t2)))
-  | ExFlo (loc, s) -> mkexp loc (Pexp_constant (ocaml_pconst_float (uv s)))
+  | ExFlo (loc, s) ->
+      mkexp loc (Pexp_constant (mkconst loc (ocaml_pconst_float (uv s))))
   | ExFor (loc, i, e1, e2, df, el) ->
       let e3 = MLast.ExSeq (loc, uv el) in
       let df = if uv df then Upto else Downto in
@@ -1238,14 +1254,7 @@ and expr =
           | _ -> error loc "bad AST"
           end
       | [PaNty (loc, s), w, e] ->
-          begin match ocaml_pexp_newtype with
-            Some newtype ->
-              begin match uv w with
-                Some _ -> error loc "(type ..) not allowed with 'when'"
-              | None -> mkexp loc (newtype (mkloc loc) (uv s) (expr e))
-              end
-          | None -> error loc "(type ..) not in this ocaml version"
-          end
+          mkexp loc (ocaml_pexp_newtype (mkloc loc) (uv s) (expr e))
       | [PaOlb (loc, p, eo), w, e] ->
           let lab = label_of_patt p in
           let (p, eo) =
@@ -1272,7 +1281,9 @@ and expr =
       in
       mkexp loc (Pexp_ifthenelse (expr e1, expr e2, e3o))
   | ExInt (loc, s, c) ->
-      mkexp loc (Pexp_constant (pconst_of_const (mkintconst loc (uv s) c)))
+      mkexp loc
+        (Pexp_constant
+           (mkconst loc (pconst_of_const (mkintconst loc (uv s) c))))
   | ExLab (loc, _) -> error loc "labeled expression not allowed here 1"
   | ExLaz (loc, e) -> mklazy loc (expr e)
   | ExLet (loc, rf, pel, e) ->
@@ -1392,8 +1403,9 @@ and expr =
   | ExStr (loc, s) ->
       mkexp loc
         (Pexp_constant
-           (ocaml_pconst_string (string_of_string_token loc (uv s))
-              (mkloc loc) None))
+           (mkconst loc
+              (ocaml_pconst_string (string_of_string_token loc (uv s))
+                 (mkloc loc) None)))
   | ExTry (loc, e, pel) ->
       mkexp loc (Pexp_try (expr e, List.map mkpwe (uv pel)))
   | ExTup (loc, el) -> mkexp loc (Pexp_tuple (List.map expr (uv el)))
