@@ -1,20 +1,12 @@
 (* camlp5r *)
 (* main.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 (* #load "q_MLast.cmo" *)
 
 open Printf;;
 open Versdep;;
 
-let string_of_loc fname line bp ep =
-  match Sys.os_type with
-    "MacOS" ->
-      sprintf "File \"%s\"; line %d; characters %d to %d\n### " fname line bp
-        ep
-  | _ -> sprintf "File \"%s\", line %d, characters %d-%d:\n" fname line bp ep
-;;
-
 let print_location loc =
   let loc =
     if Ploc.file_name loc = "" then
@@ -27,7 +19,8 @@ let print_location loc =
   if fname <> "-" then
     let line = Ploc.line_nb loc in
     let bol = Ploc.bol_pos loc in
-    eprintf "%s" (string_of_loc fname line (bp - bol + 1) (ep - bol + 1))
+    eprintf "%s"
+      (Pcaml.string_of_loc fname line (bp - bol + 1) (ep - bol + 1))
   else eprintf "At location %d-%d\n" bp ep
 ;;
 
@@ -88,15 +81,11 @@ let rec parse_file pa getdir useast =
                       let (pl, eloc) = use_file pa getdir useast s in
                       list_rev_append rpl [useast loc1 s pl, loc1]
                   | loc, x, eo ->
-                      begin match
-                        begin try Some (Pcaml.find_directive x) with
-                          Not_found -> None
-                        end
-                      with
-                        Some f -> f eo
-                      | None ->
-                          let msg = sprintf "unknown directive #%s" x in
-                          Ploc.raise loc (Stream.Error msg)
+                      begin try Pcaml.find_directive x eo with
+                        Not_found ->
+                          begin let msg = sprintf "unknown directive #%s" x in
+                            Ploc.raise loc (Stream.Error msg)
+                          end
                       end;
                       pl
                   end
@@ -138,10 +127,10 @@ let usesig loc fname ast = MLast.SgUse (loc, fname, ast);;
 let usestr loc fname ast = MLast.StUse (loc, fname, ast);;
 
 let process_intf () =
-  process !(Pcaml.parse_interf) !(Pcaml.print_interf) gind usesig
+  process Pcaml.parse_interf !(Pcaml.print_interf) gind usesig
 ;;
 let process_impl () =
-  process !(Pcaml.parse_implem) !(Pcaml.print_implem) gimd usestr
+  process Pcaml.parse_implem !(Pcaml.print_implem) gimd usestr
 ;;
 
 type file_kind = Intf | Impl;;
@@ -152,13 +141,16 @@ let file_kind_of_name name =
   else raise (Arg.Bad ("don't know what to do with " ^ name))
 ;;
 
-let print_version () =
-  eprintf "Camlp5 version %s (ocaml %s)\n" Pcaml.version
-    Pconfig.ocaml_version;
-  flush stderr;
+let print_verbose_version () =
+  printf "Camlp5 version %s (ocaml %s)\n" Pcaml.version Pcaml.ocaml_version;
+  flush stdout;
   exit 0
 ;;
 
+let print_succinct_version () =
+  printf "%s\n" Pcaml.version; flush stdout; exit 0
+;;
+
 let initial_spec_list =
   ["-intf", Arg.String (fun x -> file_kind := Intf; Pcaml.input_file := x),
    "<file>  Parse <file> as an interface, whatever its extension.";
@@ -174,7 +166,10 @@ let initial_spec_list =
    "<file> Dump quotation expander result in case of syntax error.";
    "-o", Arg.String (fun x -> Pcaml.output_file := Some x),
    "<file> Output on <file> instead of standard output.";
-   "-v", Arg.Unit print_version, "Print Camlp5 version and exit."]
+   "-version", Arg.Unit print_succinct_version,
+   "Print succinct Camlp5 version and exit.";
+   "-v", Arg.Unit print_verbose_version,
+   "Print verbose Camlp5 version and exit."]
 ;;
 
 let anon_fun x = Pcaml.input_file := x; file_kind := file_kind_of_name x;;
