@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* pcaml.mli,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 (* #load "pa_macro.cmo" *)
 
@@ -15,10 +15,26 @@ val syntax_name : string ref;;
 
 type status = Ploc.t option;;
 
+type 'a ast_transducer_t =
+  { name : string;
+    parse : (char Stream.t -> 'a) option ref;
+    transform : ('a -> 'a) option ref }
+;;
+val set_ast_parse : 'a ast_transducer_t -> (char Stream.t -> 'a) -> unit;;
+val set_ast_transform : 'a ast_transducer_t -> ('a -> 'a) -> unit;;
+val transduce : 'a ast_transducer_t -> char Stream.t -> 'a;;
+val transduce_interf :
+  ((MLast.sig_item * MLast.loc) list * status) ast_transducer_t;;
+val transduce_implem :
+  ((MLast.str_item * MLast.loc) list * status) ast_transducer_t;;
+val transduce_top_phrase : MLast.str_item option ast_transducer_t;;
+val transduce_use_file : (MLast.str_item list * bool) ast_transducer_t;;
+
+
 val parse_interf :
-  (char Stream.t -> (MLast.sig_item * MLast.loc) list * status) ref;;
+  char Stream.t -> (MLast.sig_item * MLast.loc) list * status;;
 val parse_implem :
-  (char Stream.t -> (MLast.str_item * MLast.loc) list * status) ref;;
+  char Stream.t -> (MLast.str_item * MLast.loc) list * status;;
    (** Called when parsing an interface (mli file) or an implementation
        (ml file) to build the syntax tree; the returned list contains the
        phrases (signature items or structure items) and their locations;
@@ -31,14 +47,23 @@ val parse_implem :
        default, they use the grammars entries [implem] and [interf]
        defined below. *)
 
+val parse_top_phrase : char Stream.t -> MLast.str_item option;;
+val parse_use_file : char Stream.t -> MLast.str_item list * bool;;
+
+
 val gram : Grammar.g;;
    (** Grammar variable of the OCaml language *)
 
+val attribute_body : MLast.attribute_body Grammar.Entry.e;;
 val interf : ((MLast.sig_item * MLast.loc) list * status) Grammar.Entry.e;;
 val implem : ((MLast.str_item * MLast.loc) list * status) Grammar.Entry.e;;
 val top_phrase : MLast.str_item option Grammar.Entry.e;;
 val use_file : (MLast.str_item list * bool) Grammar.Entry.e;;
+val functor_parameter : MLast.functor_parameter Grammar.Entry.e;;
 val module_type : MLast.module_type Grammar.Entry.e;;
+val longident : MLast.longid Grammar.Entry.e;;
+val longident_lident : MLast.longid_lident Grammar.Entry.e;;
+val extended_longident : MLast.longid Grammar.Entry.e;;
 val module_expr : MLast.module_expr Grammar.Entry.e;;
 val signature : MLast.sig_item list MLast.v Grammar.Entry.e;;
 val structure : MLast.str_item list MLast.v Grammar.Entry.e;;
@@ -48,23 +73,30 @@ val expr : MLast.expr Grammar.Entry.e;;
 val patt : MLast.patt Grammar.Entry.e;;
 val ipatt : MLast.patt Grammar.Entry.e;;
 val ctyp : MLast.ctyp Grammar.Entry.e;;
-val let_binding : (MLast.patt * MLast.expr) Grammar.Entry.e;;
+val let_binding :
+  (MLast.patt * MLast.expr * MLast.attributes) Grammar.Entry.e;;
 val type_decl : MLast.type_decl Grammar.Entry.e;;
+val type_extension : MLast.type_extension Grammar.Entry.e;;
+val extension_constructor : MLast.extension_constructor Grammar.Entry.e;;
 val match_case :
   (MLast.patt * MLast.expr option MLast.v * MLast.expr) Grammar.Entry.e;;
-val constructor_declaration :
-  (MLast.loc * string MLast.v * MLast.ctyp list MLast.v * MLast.ctyp option)
-    Grammar.Entry.e;;
+val constructor_declaration : MLast.generic_constructor Grammar.Entry.e;;
 val label_declaration :
-  (MLast.loc * string * bool * MLast.ctyp) Grammar.Entry.e;;
+  (MLast.loc * string * bool * MLast.ctyp * MLast.attributes) Grammar.Entry.e;;
 val with_constr : MLast.with_constr Grammar.Entry.e;;
 val poly_variant : MLast.poly_variant Grammar.Entry.e;;
 val class_sig_item : MLast.class_sig_item Grammar.Entry.e;;
 val class_str_item : MLast.class_str_item Grammar.Entry.e;;
 val class_expr : MLast.class_expr Grammar.Entry.e;;
+val class_expr_simple : MLast.class_expr Grammar.Entry.e;;
 val class_type : MLast.class_type Grammar.Entry.e;;
+val alg_attribute : MLast.attribute Grammar.Entry.e;;
+val alg_attributes : MLast.attributes Grammar.Entry.e;;
+val ext_attributes :
+  ((Ploc.t * string) option * MLast.attributes_no_anti) Grammar.Entry.e;;
    (** Some entries of the language, set by [pa_o.cmo] and [pa_r.cmo]. *)
 
+
 val input_file : string ref;;
    (** The file currently being parsed. *)
 val output_file : string option ref;;
@@ -81,11 +113,27 @@ val quotation_location : unit -> Ploc.t;;
        [Failure] if not in the context of a quotation expander. *)
 val version : string;;
    (** The current version of Camlp5. *)
+val ocaml_version : string;;
+   (** The current version of OCaml, possibly truncated after space or '+':
+       e.g. if OCaml version is "4.05.0+beta3", it is "4.05.0" *)
 val add_option : string -> Arg.spec -> string -> unit;;
    (** Add an option to the command line options. *)
 val no_constructors_arity : bool ref;;
    (** [True]: dont generate constructor arity. *)
+val string_of_loc : string -> int -> int -> int -> string;;
+   (** [string_of_loc fname line bp ep] returns the location string for
+       file [fname] at [line] and between character [bp] and [ep]. *)
 
+
+type err_ctx =
+    Finding
+  | Expanding
+  | ParsingResult of Ploc.t * string
+;;
+exception Qerror of string * string * err_ctx * exn;;
+
+val expand_quotation :
+  Ploc.t -> (string -> 'b) -> int -> string -> string -> 'b;;
 val handle_expr_quotation : MLast.loc -> string * string -> MLast.expr;;
 val handle_patt_quotation : MLast.loc -> string * string -> MLast.patt;;
 
@@ -101,6 +149,7 @@ val pr_patt : MLast.patt Eprinter.t;;
 val pr_ctyp : MLast.ctyp Eprinter.t;;
 val pr_str_item : MLast.str_item Eprinter.t;;
 val pr_sig_item : MLast.sig_item Eprinter.t;;
+val pr_longident : MLast.longid Eprinter.t;;
 val pr_module_expr : MLast.module_expr Eprinter.t;;
 val pr_module_type : MLast.module_type Eprinter.t;;
 val pr_class_sig_item : MLast.class_sig_item Eprinter.t;;
@@ -149,6 +198,7 @@ val strict_mode : bool ref;;
 
 val unvala : 'a -> 'a;;
 val vala_map : ('a -> 'b) -> 'a -> 'b;;
+val vala_it : ('a -> unit) -> 'a -> unit;;
 val vala_mapa : ('a -> 'b) -> (string -> 'b) -> 'a -> 'b;;
 
 (**/**)
@@ -166,4 +216,4 @@ val expr_reloc : (MLast.loc -> MLast.loc) -> int -> MLast.expr -> MLast.expr;;
 val rename_id : (string -> string) ref;;
 val flag_comments_in_phrases : bool ref;;
 val flag_equilibrate_cases : bool ref;;
-val flag_compatible_old_versions_of_ocaml : bool ref;;
+val flag_expand_letop_syntax : bool ref;;
