@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* prtools.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 (* #load "q_MLast.cmo" *)
 (* #load "pa_macro.cmo" *)
@@ -15,12 +15,6 @@ type pr_context =
 
 type 'a pr_fun = pr_context -> 'a -> string;;
 
-let with_ind_bef = Pprintf.with_ind_bef;;
-let with_bef = Pprintf.with_bef;;
-let with_bef_aft_dang = Pprintf.with_bef_aft_dang;;
-let with_aft = Pprintf.with_aft;;
-let with_aft_dang = Pprintf.with_aft_dang;;
-
 let tab ind = String.make ind ' ';;
 
 (* horizontal list *)
@@ -29,8 +23,8 @@ let rec hlist elem pc xl =
     [] -> sprintf "%s%s" pc.bef pc.aft
   | [x] -> elem pc x
   | x :: xl ->
-      sprintf "%s %s" (elem (with_aft_dang pc "" "") x)
-        (hlist elem (with_bef pc "") xl)
+      sprintf "%s %s" (elem {pc with aft = ""; dang = ""} x)
+        (hlist elem {pc with bef = ""} xl)
 ;;
 
 (* horizontal list with different function from 2nd element on *)
@@ -39,8 +33,8 @@ let rec hlist2 elem elem2 pc xl =
     [] -> invalid_arg "hlist2"
   | [x] -> elem pc x
   | x :: xl ->
-      sprintf "%s %s" (elem (with_aft pc "") x)
-        (hlist2 elem2 elem2 (with_bef pc "") xl)
+      sprintf "%s %s" (elem {pc with aft = ""} x)
+        (hlist2 elem2 elem2 {pc with bef = ""} xl)
 ;;
 
 (* horizontal list with different function for the last element *)
@@ -49,8 +43,8 @@ let rec hlistl elem eleml pc xl =
     [] -> sprintf "%s%s" pc.bef pc.aft
   | [x] -> eleml pc x
   | x :: xl ->
-      sprintf "%s %s" (elem (with_aft_dang pc "" "") x)
-        (hlistl elem eleml (with_bef pc "") xl)
+      sprintf "%s %s" (elem {pc with aft = ""; dang = ""} x)
+        (hlistl elem eleml {pc with bef = ""} xl)
 ;;
 
 (* vertical list *)
@@ -59,8 +53,8 @@ let rec vlist elem pc xl =
     [] -> sprintf "%s%s" pc.bef pc.aft
   | [x] -> elem pc x
   | x :: xl ->
-      sprintf "%s\n%s" (elem (with_aft_dang pc "" "") x)
-        (vlist elem (with_bef pc (tab pc.ind)) xl)
+      sprintf "%s\n%s" (elem {pc with aft = ""; dang = ""} x)
+        (vlist elem {pc with bef = tab pc.ind} xl)
 ;;
 
 (* vertical list with different function from 2nd element on *)
@@ -69,8 +63,8 @@ let rec vlist2 elem elem2 pc xl =
     [] -> invalid_arg "vlist2"
   | [x] -> elem pc x
   | x :: xl ->
-      sprintf "%s\n%s" (elem (with_aft pc "") x)
-        (vlist2 elem2 elem2 (with_bef pc (tab pc.ind)) xl)
+      sprintf "%s\n%s" (elem {pc with aft = ""} x)
+        (vlist2 elem2 elem2 {pc with bef = tab pc.ind} xl)
 ;;
 
 (* vertical list with different function from 2nd element on *)
@@ -79,8 +73,8 @@ let rec vlist3 elem elem2 pc xl =
     [] -> invalid_arg "vlist3"
   | [x] -> elem pc (x, true)
   | x :: xl ->
-      sprintf "%s\n%s" (elem (with_aft pc "") (x, false))
-        (vlist3 elem2 elem2 (with_bef pc (tab pc.ind)) xl)
+      sprintf "%s\n%s" (elem {pc with aft = ""} (x, false))
+        (vlist3 elem2 elem2 {pc with bef = tab pc.ind} xl)
 ;;
 
 (* vertical list with different function for the last element *)
@@ -89,8 +83,8 @@ let rec vlistl elem eleml pc xl =
     [] -> sprintf "%s%s" pc.bef pc.aft
   | [x] -> eleml pc x
   | x :: xl ->
-      sprintf "%s\n%s" (elem (with_aft_dang pc "" "") x)
-        (vlistl elem eleml (with_bef pc (tab pc.ind)) xl)
+      sprintf "%s\n%s" (elem {pc with aft = ""; dang = ""} x)
+        (vlistl elem eleml {pc with bef = tab pc.ind} xl)
 ;;
 
 (* vertical list applied to a list of functions *)
@@ -99,8 +93,8 @@ let rec vlistf pc fl =
     [] -> sprintf "%s%s" pc.bef pc.aft
   | [f] -> f pc
   | f :: fl ->
-      sprintf "%s\n%s" (f (with_aft_dang pc "" ""))
-        (vlistf (with_bef pc (tab pc.ind)) fl)
+      sprintf "%s\n%s" (f {pc with aft = ""; dang = ""})
+        (vlistf {pc with bef = tab pc.ind} fl)
 ;;
 
 let hvlistl elem eleml pc xl =
@@ -113,11 +107,9 @@ let rise_string ind sh b s =
      double-quote) which contains newlines, attempt to concat its first
      line in the previous line, and, instead of displaying this:
               eprintf
-                "\
-           hello, world"
+                "hello, world"
      displays that:
-              eprintf "\
-           hello, world"
+              eprintf "hello, world"
      what "saves" one line.
    *)
   if String.length s > ind + sh && s.[ind+sh] = '"' then
@@ -152,26 +144,27 @@ and plistl_two_parts elem eleml sh pc xl =
   | [x, _] -> eleml pc x, None
   | (x, sep) :: xl ->
       let s =
-        horiz_vertic (fun () -> Some (elem (with_aft_dang pc sep sep) x))
+        horiz_vertic (fun () -> Some (elem {pc with aft = sep; dang = sep} x))
           (fun () -> None)
       in
       match s with
-        Some b -> plistl_kont_same_line elem eleml sh (with_bef pc b) xl, None
+        Some b ->
+          plistl_kont_same_line elem eleml sh {pc with bef = b} xl, None
       | None ->
-          let s1 = elem (with_aft_dang pc sep sep) x in
+          let s1 = elem {pc with aft = sep; dang = sep} x in
           let s2 =
             plistl elem eleml 0
-              (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh))) xl
+              {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)} xl
           in
           s1, Some s2
 and plistl_kont_same_line elem eleml sh pc xl =
   match xl with
     [] -> assert false
   | [x, _] ->
-      horiz_vertic (fun () -> eleml (with_bef pc (sprintf "%s " pc.bef)) x)
+      horiz_vertic (fun () -> eleml {pc with bef = sprintf "%s " pc.bef} x)
         (fun () ->
            let s =
-             eleml (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh))) x
+             eleml {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)} x
            in
            let (b, s) = rise_string pc.ind sh pc.bef s in
            sprintf "%s\n%s" b s)
@@ -180,15 +173,17 @@ and plistl_kont_same_line elem eleml sh pc xl =
         horiz_vertic
           (fun () ->
              Some
-               (elem (with_bef_aft_dang pc (sprintf "%s " pc.bef) sep sep) x))
+               (elem
+                  {pc with bef = sprintf "%s " pc.bef; aft = sep; dang = sep}
+                  x))
           (fun () -> None)
       in
       match s with
-        Some b -> plistl_kont_same_line elem eleml sh (with_bef pc b) xl
+        Some b -> plistl_kont_same_line elem eleml sh {pc with bef = b} xl
       | None ->
           let (s1, s2o) =
             plistl_two_parts elem eleml 0
-              (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh)))
+              {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)}
               ((x, sep) :: xl)
           in
           match s2o with
@@ -207,10 +202,10 @@ let plistb elem sh pc xl =
   match xl with
     [] -> sprintf "%s%s" pc.bef pc.aft
   | [x, _] ->
-      horiz_vertic (fun () -> elem (with_bef pc (sprintf "%s " pc.bef)) x)
+      horiz_vertic (fun () -> elem {pc with bef = sprintf "%s " pc.bef} x)
         (fun () ->
            let s =
-             elem (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh))) x
+             elem {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)} x
            in
            sprintf "%s\n%s" pc.bef s)
   | (x, sep) :: xl ->
@@ -218,11 +213,13 @@ let plistb elem sh pc xl =
         horiz_vertic
           (fun () ->
              Some
-               (elem (with_bef_aft_dang pc (sprintf "%s " pc.bef) sep sep) x))
+               (elem
+                  {pc with bef = sprintf "%s " pc.bef; aft = sep; dang = sep}
+                  x))
           (fun () -> None)
       in
       match s with
-        Some b -> plistl_kont_same_line elem elem sh (with_bef pc b) xl
+        Some b -> plistl_kont_same_line elem elem sh {pc with bef = b} xl
       | None ->
           let s1 =
             let s =
@@ -235,7 +232,7 @@ let plistb elem sh pc xl =
           in
           let s2 =
             plistl elem elem 0
-              (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh))) xl
+              {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)} xl
           in
           sprintf "%s\n%s" s1 s2
 ;;
@@ -255,39 +252,42 @@ and plistf_two_parts sh pc xl =
   | [x, _] -> x pc, None
   | (x, sep) :: xl ->
       let s =
-        horiz_vertic (fun () -> Some (x (with_aft_dang pc sep sep)))
+        horiz_vertic (fun () -> Some (x {pc with aft = sep; dang = sep}))
           (fun () -> None)
       in
       match s with
-        Some b -> plistf_kont_same_line sh (with_bef pc b) xl, None
+        Some b -> plistf_kont_same_line sh {pc with bef = b} xl, None
       | None ->
-          let s1 = x (with_aft_dang pc sep sep) in
+          let s1 = x {pc with aft = sep; dang = sep} in
           let s2 =
-            plistf 0 (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh))) xl
+            plistf 0 {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)} xl
           in
           s1, Some s2
 and plistf_kont_same_line sh pc xl =
   match xl with
     [] -> assert false
   | [x, _] ->
-      horiz_vertic (fun () -> x (with_bef pc (sprintf "%s " pc.bef)))
+      horiz_vertic (fun () -> x {pc with bef = sprintf "%s " pc.bef})
         (fun () ->
-           let s = x (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh))) in
+           let s = x {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)} in
            let (b, s) = rise_string pc.ind sh pc.bef s in
            sprintf "%s\n%s" b s)
   | (x, sep) :: xl ->
       let s =
         horiz_vertic
           (fun () ->
-             Some (x (with_bef_aft_dang pc (sprintf "%s " pc.bef) sep sep)))
+             Some
+               (x
+                  {pc with bef = sprintf "%s " pc.bef; aft = sep;
+                   dang = sep}))
           (fun () -> None)
       in
       match s with
-        Some b -> plistf_kont_same_line sh (with_bef pc b) xl
+        Some b -> plistf_kont_same_line sh {pc with bef = b} xl
       | None ->
           let (s1, s2o) =
             plistf_two_parts 0
-              (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh)))
+              {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)}
               ((x, sep) :: xl)
           in
           match s2o with
@@ -303,19 +303,22 @@ let rec plistbf sh pc xl =
   match xl with
     [] -> sprintf "%s%s" pc.bef pc.aft
   | [x, _] ->
-      horiz_vertic (fun () -> x (with_bef pc (sprintf "%s " pc.bef)))
+      horiz_vertic (fun () -> x {pc with bef = sprintf "%s " pc.bef})
         (fun () ->
-           let s = x (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh))) in
+           let s = x {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)} in
            sprintf "%s\n%s" pc.bef s)
   | (x, sep) :: xl ->
       let s =
         horiz_vertic
           (fun () ->
-             Some (x (with_bef_aft_dang pc (sprintf "%s " pc.bef) sep sep)))
+             Some
+               (x
+                  {pc with bef = sprintf "%s " pc.bef; aft = sep;
+                   dang = sep}))
           (fun () -> None)
       in
       match s with
-        Some b -> plistf_kont_same_line sh (with_bef pc b) xl
+        Some b -> plistf_kont_same_line sh {pc with bef = b} xl
       | None ->
           let s1 =
             let s =
@@ -326,7 +329,7 @@ let rec plistbf sh pc xl =
             sprintf "%s\n%s" pc.bef s
           in
           let s2 =
-            plistf 0 (with_ind_bef pc (pc.ind + sh) (tab (pc.ind + sh))) xl
+            plistf 0 {pc with ind = pc.ind + sh; bef = tab (pc.ind + sh)} xl
           in
           sprintf "%s\n%s" s1 s2
 ;;
@@ -337,8 +340,8 @@ module Buff =
   struct
     let buff = ref (string_create 80);;
     let store len x =
-      if len >= String.length !buff then
-        buff := !buff ^ string_create (String.length !buff);
+      if len >= string_length !buff then
+        buff := string_cat !buff (string_create (string_length !buff));
       string_set !buff len x;
       succ len
     ;;
@@ -349,7 +352,7 @@ module Buff =
       in
       add_rec len 0
     ;;
-    let get len = String.sub !buff 0 len;;
+    let get len = string_sub !buff 0 len;;
   end
 ;;
 
@@ -400,7 +403,7 @@ let adjust_comment_indentation ind s nl_bef ind_bef =
         in
         loop olen i
     in
-    loop olen 0
+    bytes_to_string (loop olen 0)
 ;;
 
 let eight_chars = String.make 8 ' ';;
@@ -410,7 +413,7 @@ let expand_tabs s =
     else if s.[i] = '\t' then loop (Buff.mstore len eight_chars) (i + 1)
     else loop (Buff.store len s.[i]) (i + 1)
   in
-  loop 0 0
+  bytes_to_string (loop 0 0)
 ;;
 
 let comm_bef ind loc =
@@ -427,10 +430,10 @@ let expand_module_prefix m =
       (p, e) :: rest ->
         let p =
           match p with
-            MLast.PaAcc (_, MLast.PaUid (_, _), _) -> p
+            MLast.PaPfx (_, MLast.LiUid (_, _), _) -> p
           | _ ->
               let loc = MLast.loc_of_patt p in
-              MLast.PaAcc (loc, MLast.PaUid (loc, m), p)
+              MLast.PaPfx (loc, MLast.LiUid (loc, m), p)
         in
         loop ((p, e) :: rev_lel) rest
     | [] -> List.rev rev_lel
@@ -482,30 +485,3 @@ let do_split_or_patterns_with_bindings pel =
   in
   loop [] pel
 ;;
-
-let record_without_with loc e lel =
-  try
-    let (m, name) =
-      let (m, sl) =
-        List.fold_right
-          (fun (p, _) (m, sl) ->
-             match p with
-               MLast.PaLid (_, lab) -> m, lab :: sl
-             | MLast.PaAcc (_, MLast.PaUid (_, m), MLast.PaLid (_, lab)) ->
-                 m, lab :: sl
-             | _ -> raise Exit)
-          lel ("", [])
-      in
-      m, String.concat "_" ("with" :: sl)
-    in
-    let f =
-      let f = MLast.ExLid (loc, name) in
-      if m = "" then f else MLast.ExAcc (loc, MLast.ExUid (loc, m), f)
-    in
-    let e =
-      List.fold_left (fun e1 (_, e2) -> MLast.ExApp (loc, e1, e2))
-        (MLast.ExApp (loc, f, e)) lel
-    in
-    Some e
-  with Exit -> None
-;;
