@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* odyl.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 let apply_load () =
   let i = ref 1 in
@@ -37,4 +37,4 @@ let main () =
       exit 2
 ;;
 
-Printexc.catch main ();;
+main ();;
