@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* odyl.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 value apply_load () =
   let i = ref 1 in
@@ -38,4 +38,4 @@ value main () =
       } ]
 ;
 
-Printexc.catch main ();
+main ();
