@@ -105,7 +105,12 @@ value print_out_value ppf tree =
     | Oval_tuple tree_list ->
         fprintf ppf "@[<1>(%a)@]" (print_tree_list print_tree ",") tree_list
     | Oval_ellipsis -> raise Ellipsis
-    | Oval_printer f -> f ppf
+    | Oval_printer f ->
+       IFDEF OCAML_VERSION < OCAML_5_3_0 THEN
+         f ppf
+       ELSE
+         Format.fprintf ppf "%s" (Format_doc.asprintf "%a" (fun ppf () -> f ppf) ())
+       END
     | tree -> fprintf ppf "@[<1>(%a)@]" (cautious print_tree) tree ]
   and print_fields first ppf =
     fun
@@ -210,6 +215,13 @@ value rec print_out_type ppf =
 and print_out_type_1 ppf =
   fun
   [ Otyp_arrow lab ty1 ty2 ->
+    let lab =
+      IFDEF OCAML_VERSION < OCAML_5_2_0 THEN
+      lab
+      ELSE
+      match lab with [ Nolabel -> "" | Labelled s -> s | Optional s -> s ]
+      END
+    in
       fprintf ppf "@[%s%a %s@ %a@]" (if lab <> "" then lab ^ ":" else "")
         print_out_type_2 ty1 (if utf8 then "→" else "->") print_out_type_1 ty2
   | ty -> print_out_type_2 ppf ty ]
@@ -367,14 +379,27 @@ and print_out_constr_gadt_opt ppf = fun [
  {ocstr_name=name; ocstr_args=tyl; ocstr_return_type=rto} ->
   match rto with
   [ Some rt ->
-      let t = List.fold_right (fun t1 t2 -> Otyp_arrow "" t1 t2) tyl rt in
+    let lab =
+    IFDEF OCAML_VERSION < OCAML_5_2_0 THEN
+      ""
+    ELSE
+      Asttypes.Nolabel
+    END in
+      let t = List.fold_right (fun t1 t2 -> Otyp_arrow lab t1 t2) tyl rt in
       fprintf ppf "%s : %a" name print_out_type t
   | None -> print_out_constr ppf (name, tyl) ]
   END
 ]
-and print_out_label ppf (name, mut, arg) =
-  fprintf ppf "@[<2>%s :@ %s%a@]" name (if mut then "mutable " else "")
-    print_out_type arg
+and print_out_label ppf l =
+  IFDEF OCAML_VERSION < OCAML_5_3_0 THEN
+  let (name, mut, arg) = l in
+      fprintf ppf "@[<2>%s :@ %s%a@]" name (if mut then "mutable " else "")
+        print_out_type arg
+  ELSE
+  let {olab_name=name; olab_mut=mut; olab_type=arg} = l in
+      fprintf ppf "@[<2>%s :@ %s%a@]" name (if mut=Mutable then "mutable " else "")
+        print_out_type arg
+  END
 and print_fields rest ppf =
   fun
   [ [] ->
@@ -425,12 +450,18 @@ and print_typargs ppf =
       fprintf ppf "@[<1>(%a)@]@ " (print_typlist print_out_type ",") tyl ]
 ;
 
+IFDEF OCAML_VERSION < OCAML_5_2_0 THEN
+value print_out_class_param ppf (x, _) = fprintf ppf "'%s" x ;
+ELSE
+value print_out_class_param ppf p = fprintf ppf "'%s" p.ot_name ;
+END ;
+
 value print_out_class_params ppf =
   fun
   [ [] -> ()
   | tyl ->
       fprintf ppf "@[<1>[%a]@]@ "
-        (print_list (fun ppf (x, _) -> fprintf ppf "'%s" x)
+        (print_list print_out_class_param
            (fun ppf -> fprintf ppf ", "))
         tyl ]
 ;
@@ -449,6 +480,13 @@ value rec print_out_class_type ppf =
       in
       fprintf ppf "@[%a%a@]" pr_tyl tyl print_ident id
   | Octy_arrow lab ty cty ->
+     let lab =
+       IFDEF OCAML_VERSION < OCAML_5_2_0 THEN
+       lab
+       ELSE
+       match lab with [ Nolabel -> "" | Labelled s -> s | Optional s -> s ]
+       END
+     in
       fprintf ppf "@[%s[ %a ] ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
         print_out_type ty print_out_class_type cty
   | Octy_signature self_ty csil ->
@@ -604,7 +642,14 @@ and print_out_type_decl kwd ppf x =
       print_out_type ty'
   in
   let print_constraints ppf params = List.iter (constrain ppf) params in
-  let type_parameter ppf (ty, var_inj) =
+  let type_parameter ppf otp =
+    let (ty, var_inj) =
+    IFDEF OCAML_VERSION < OCAML_5_2_0 THEN
+      otp
+    ELSE
+      (otp.ot_name, otp.ot_variance)
+    END
+    in
     let (q, ty) = try_greek ty in
     IFDEF OCAML_VERSION < OCAML_4_12_0 THEN
     let (co, cn) = var_inj in
@@ -669,18 +714,31 @@ value print_out_phrase ppf =
 value recover_from_print_failure ~{fname} f fallbackf ppf v =
   try f ppf v
   with e -> do {
-    Format.fprintf ppf "%!\n================\n%!EXN %s\nPlease report this error\n%!" (Printexc.to_string e) ;
+    IFDEF OCAML_VERSION < OCAML_5_3_0 THEN
+      Format.fprintf ppf "%!\n================\n%!EXN %s\nPlease report this error\n%!" (Printexc.to_string e)
+    ELSE
+      Format_doc.fprintf ppf "%!\n================\n%!EXN %s\nPlease report this error\n%!" (Printexc.to_string e)
+    END ;
     fallbackf ppf v
   }
 ;
 
+IFDEF OCAML_VERSION < OCAML_5_3_0 THEN
+value format_to_format_doc__printer pfun ppf t = pfun ppf t ;
+ELSE
+value format_to_format_doc__printer pfun ppf t =
+  let s = Format.asprintf "%a" pfun t in
+  Format_doc.fprintf ppf "%s" s ;
+END
+;
+
 Toploop.print_out_value.val := print_out_value;
-Toploop.print_out_type.val := print_out_type;
-  Toploop.print_out_class_type.val := print_out_class_type;
+Toploop.print_out_type.val := format_to_format_doc__printer print_out_type;
+  Toploop.print_out_class_type.val := format_to_format_doc__printer print_out_class_type;
   Toploop.print_out_module_type.val :=
-  recover_from_print_failure ~{fname="print_out_module_type"} print_out_module_type Toploop.print_out_module_type.val;
+  recover_from_print_failure ~{fname="print_out_module_type"} (format_to_format_doc__printer print_out_module_type) Toploop.print_out_module_type.val;
 Toploop.print_out_signature.val :=
-  recover_from_print_failure ~{fname="print_out_signature"} print_out_signature Pp_outcometree.pp_out_sig_item_list;
+  recover_from_print_failure ~{fname="print_out_signature"} (format_to_format_doc__printer print_out_signature) (format_to_format_doc__printer Pp_outcometree.pp_out_sig_item_list);
   Toploop.print_out_sig_item.val :=
-  recover_from_print_failure ~{fname="print_out_sig_item"} print_out_sig_item Toploop.print_out_sig_item.val;
+  recover_from_print_failure ~{fname="print_out_sig_item"} (format_to_format_doc__printer print_out_sig_item) Toploop.print_out_sig_item.val;
 Toploop.print_out_phrase.val := print_out_phrase;
